#!/usr/bin/perl -w

##  Copyright 2004 Rien Croonenborghs
##
##    This file is part of file2divx3pass.
##    file2divx4pass is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##    file2divx3pass is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    You should have received a copy of the GNU General Public License
##    along with file2divx3pass; if not, write to the Free Software
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package File2DiVX3Pass2;
use strict;

my $errormessage = '';
my %hfilespecs = ( 	'length' => 0,
				'fps' => 0,
				'videobitrate' => 0,
				'audiobitrate' => 0	);

my @mp3bitrates = ( 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320 );
my %settings = (	'subtype' => 'no',
				'sid' => 1,
				'audiocodec' => 'MP3',
				'mp3rate' => 'constant',
				'mp3bitrate' => 128,
				'mp3bitratesaved' => 128,
				'aidset' => FALSE,
				'aid' => 1,
				'cdsize' => '700',
				'fpsset' => FALSE,
				'ofps' => 25,
				'scalew' => 480,
				'scaleh' => 360,
				'scaleratio' => '43',
				'videocodec' => 'lavc',
				'lavcopts' => 'vcodec=mpeg4:vhq:vqmin=2:vqmax=31:keyint=25',
				'divx4opts' => 'q=5:min_quant=2:max_quant=31',
				'passes' => 1,
				'filters' => 'pp=de',
				'mplayerpath' => '/usr/bin/mplayer',
				'mencoderpath' => '/usr/bin/mencoder',
				'cleanup' => 'before+after',
				'debug' => FALSE,
				'isencodingstatus' => FALSE,
				'isbatchencodingstatus' => FALSE,
				'mencoderpid' => -1								);

## main ------------------------------------------------------------------------

## get the array with mp3 bitrates
sub getMP3Bitrates
{
	return @mp3bitrates
}

## set setting
sub setSetting
{
	my ($key,$value) = @_;
	$settings{ $key } = $value;
}
## get setting
sub getSetting
{
	my ($key) = @_;
	return $settings{ $key };
}
## check if settings dir in home dir exists
sub settingsDirExists
{
	## get homedir
	my $home = $ENV{HOME};
	## check if .file2divx3pass dir exists
	if( -e $home."/.file2divx3pass" && -d $home."/.file2divx3pass" )
	{ return TRUE; }
	else { return FALSE; }
}

## set an error
sub setError
{
	my ($errormsg) = @_;
	$errormessage = $errormsg;
}
## get the error
sub getError
{
	return $errormessage;
}

## print an error in the debug section
sub printError
{
	my ($widgets,$errormsg) = @_;

	$widgets->{'lblDebug'}->set_text( '[Error] '.$errormsg );

	my $buffer = $widgets->{'tvMainDebug'}->get_buffer;
	$buffer->insert_at_cursor( getCurrentDateString().' || [Error] '.$errormsg."\n" );
	$widgets->{'tvMainDebug'}->set_buffer( $buffer );
}
## print a normal message in the debug section
sub printMsg
{
	my ($widgets,$msg) = @_;

	$widgets->{'lblDebug'}->set_text( $msg );

	my $buffer = $widgets->{'tvMainDebug'}->get_buffer;
	$buffer->insert_at_cursor( getCurrentDateString().' || '.$msg."\n" );
	$widgets->{'tvMainDebug'}->set_buffer( $buffer );
}


## FILE TOOLS ------------------------------------------------------------------

## identify the input file with mplayer
## get the length + FPS + video bitrate
sub readFileSpecs
{
	my ($file,$widgets) = @_;

	## clear the previous specs
	%hfilespecs = ( 'length' => 0,
				 'fps' => 0,
				 'videobitrate' => 0,
				 'audiobitrate' => 0 );

	## mplayer path + check if it exists
	my $mplayer = $widgets->{'entryProgramsMPlayer'}->get_text;
	if( !-e $mplayer )
	{
		printErrro( $widgets, "Cannot find mplayer at $mplayer" );
		return;
	}

	## prepare cmd to be run
	my $cmd = "$mplayer -identify -vc null -ac null -frames 0 \"$file\"";
	printMsg( $widgets, $cmd );
	## open
	my $res = open( TMP, "$cmd 2>&1 |" );
	if( !defined($res) )
	{
		setError( 'Could not identify '.$file );
		return FALSE;
	}
	## scan
	while( <TMP> )
	{
		## frames/s
		if( $_ =~ /ID_VIDEO_FPS=(\d+\.\d+)/ )
		{
			$hfilespecs{'fps'} = $1;
			printMsg( $widgets, "Found FPS $1" );
		}
		## some locales use the comma as decimal symbol
		elsif( $_ =~ /ID_VIDEO_FPS=(\d+\,\d+)/ )
		{
			$hfilespecs{'fps'} = $1;
			$hfilespecs{'fps'} =~ s/\,/\./g;
			printMsg( $widgets, "Found FPS $1" );
		}
		## length
		## realmedia will return 0 no mather what -> so indicate with 59:59:59 time
		elsif( $_ =~ /ID_LENGTH=(\d+)/ )
		{
			$hfilespecs{'length'} = $1;
			if( $hfilespecs{'length'} eq "0" ) { $hfilespecs{'length'} = "215999"; }
			printMsg( $widgets, "Found video length $1" );
		}
		elsif( $_ =~ /ID_VIDEO_BITRATE=(\d+)/ )
		{
			$hfilespecs{'videobitrate'} = $1;
			printMsg( $widgets, "Found video bitrate ".($1/1000) );
		}
		elsif( $_ =~ /ID_AUDIO_BITRATE=(\d+)/ )
		{
			$hfilespecs{'audiobitrate'} = $1;
			printMsg( $widgets, "Found audio bitrate ".($1/1000) );
		}
	}
	## close
	close( TMP );

	## check the fps and seconds
	if( !defined($hfilespecs{'fps'}) )
	{
		setError( 'No fps found; make sure this is a valid media file' );
		return FALSE;
	}
	elsif( !defined($hfilespecs{'length'}) )
	{
		setError( 'No length found; make sure this is a valid media file' );
		return FALSE;
	}
	return TRUE;
}
sub getFileSpec
{
	my ($key) = @_;
	return $hfilespecs{ $key };
}

## CALCULATE THINGS :) ---------------------------------------------------------

## calc video bitrate
sub calculateVideoBrate
{
	my ($cdsize,$minutes,$audiobrate) = @_;
	if( !$minutes || $minutes == 0 ) { return 0; }
	my $videobrate = ( ( ( ( $cdsize * 1024 * 8 ) / 1000 ) * 1024 ) - ( $audiobrate * $minutes * 60 ) ) / ( $minutes * 60 );
	$videobrate = substr( $videobrate, 0, index( $videobrate, "." ) );
	# taken from divxcalc 0.5 (http://axljab.homelinux.org:8080/?m=sw&p=divxcalc)
	return $videobrate;
}

sub calculateVideoBrate4GUI
{
	my ($widgets) = @_;

	## get video length
	my $seconds = getFileSpec( 'length' );
	## check the size of the CD
	my $cdsize = $widgets->{'entryVideoCDSize'}->get_text;
	if( !txtIsNumeral( $cdsize ) )
	{
		printError( $widgets, getError().' for CD Size') ;
		return -1;
	}
	## get the audio bitrate
	my $audiobrate = getSetting( 'mp3bitrate' );
	if( getSetting( 'audiocodec' ) eq 'copy' ) { $audiobrate = getFileSpec( 'audiobitrate' ); }
	## get fps
	my $fps = getFileSpec( 'fps' );;
	## calculate video bitrate
	my $videobrate = calculateVideoBrate( $cdsize, $seconds/60, $audiobrate );

	return $videobrate;
}

## calc Bits/Pixel
sub calculateBitsPerPixel
{
	my ($videobrate,$width,$height,$fps) = @_;
	if( !$videobrate || $videobrate == 0 ) { return 0; }
	$fps  =~ s/\,/\./g; ## locales decimal
	my $bpp = ( $videobrate * 1000 ) / ( $width * $height * $fps );
	return $bpp;
}

## CHECK ENTRIES ---------------------------------------------------------------

## check if text has only numeral [0-9]
sub txtIsNumeral
{
	my ($txt) = @_;
	## no empty line
	if( $txt eq "" )
	{ setError( 'No value found' ); return FALSE; }
	## only 0-9
	elsif( $txt =~ /[A-Za-z]/ || $txt =~ /.,;:=?\/+~%'`\[\]\$\*&\"\'(!)-_|@#{}<>\\/ )
	{ setError( 'Only numerals (0-9)' ); return FALSE; }
	return TRUE;
}
## check if text has only numeral [0-9] and decimal
sub txtIsNumeralDecimal
{
	my ($txt) = @_;
	## no empty line
	if( $txt eq "" )
	{ setError( 'No value found' ); return FALSE; }
	## only 0-9
	elsif( $txt =~ /[A-Za-z]/ || $txt =~ /;:=?\/+~%'`\[\]\$\*&\"\'(!)-_|@#{}<>\\/ )
	{ setError( 'Only numerals (0-9 or decimal)' ); return FALSE; }
	return TRUE;
}
## check if text has [A-Za-z0-9] characters + other chars
sub txtIsNonNumeral
{
	my ($txt) = @_;
	## no empty line
	if( $txt eq "" )
	{ setError( 'No value found' ); return FALSE; }
	return TRUE;
}
## check if text is hh:mm:ss formatted hour
sub txtIsHour
{
	my ($txt) = @_;
	## no empty line
	if( $txt eq "" )
	{ setError( 'No value found' ); return FALSE; }
	my ($h,$m,$s) = split( /:/, $txt );

	if( !( $h =~ /([0-9]{2})/ ) ) { setError( 'Wrong hour value' ); return FALSE; }
	if( !( $m =~ /([0-9]{2})/ ) ) { setError( 'Wrong minute value' ); return FALSE; }
	if( !( $s =~ /([0-9]{2})/ ) ) { setError( 'Wrong second value' ); return FALSE; }

	return TRUE;
}


## HOUR<->SECONDS --------------------------------------------------------------

## hh:mm:ss to seconds
sub HtoS
{
	my ($h) = @_;

	my ($hh,$mm,$ss) = split( /:/, $h );
	return ($hh*3600) + ($mm*60) + $ss;
}

## seconds to hh:mm:ss
sub StoH
{
	my ($s) = @_;

	if( $s < 0 ) { return "neg. time"; }

	my $hh = 0;
	my $mm = 0;

	if( $s / 3600 > 0 )
	{
		$hh = $s / 3600;
		$hh = substr( $hh, 0, index($hh,".") ) if( $hh =~ /\./ );
		$s = $s % 3600;
	}
	if( $s / 60 > 0 )
	{
		$mm = $s / 60;
		$mm = substr( $mm, 0, index($mm,".") ) if( $mm =~ /\./ );
		$s = $s % 60;
	}
	for( my $i=length($hh); $i<2; ++$i ) { $hh = "0".$hh; }
	for( my $i=length($mm); $i<2; ++$i ) { $mm = "0".$mm; }
	for( my $i=length($s); $i<2; ++$i ) { $s = "0".$s; }
	return "$hh:$mm:$s";
}

## get date string
sub getCurrentDateString
{
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	++$mon; $year += 1900;
	for( my $i=length($wday); $i<2; ++$i ) { $wday = '0'.$wday; }
	for( my $i=length($mon); $i<2; ++$i ) { $mon = '0'.$mon; }
	for( my $i=length($year); $i<2; ++$i ) { $year = '0'.$year; }
	for( my $i=length($hour); $i<2; ++$i ) { $hour = '0'.$hour; }
	for( my $i=length($min); $i<2; ++$i ) { $min = '0'.$min; }
	for( my $i=length($sec); $i<2; ++$i ) { $sec = '0'.$sec; }

	my $datestr = "$wday/$mon/$year $hour:$min:$sec";
	return $datestr;
}


## ENCODING --------------------------------------------------------------------

## ENCODE VIDEO : prepare the command(s) to runs
sub encodeVideo
{
	my ( $inputfile,
		$outputfile,
		$widgets )   = @_;

	## make audio options
	my $cmdaudio = '';
	if( getSetting( 'audiocodec' ) eq 'MP3' )
	{
		$cmdaudio = ' -oac mp3lame -lameopts ';
		## rate
		if( getSetting( 'mp3rate' ) eq 'constant' ) { $cmdaudio .= 'cbr:'; }
		elsif( getSetting( 'mp3rate' ) eq 'average' ) { $cmdaudio .= 'abr:'; }
		## bitrate
		$cmdaudio .= 'br='.getSetting( 'mp3bitrate' ).' ';
	}
	elsif( getSetting( 'audiocodec' ) eq 'copy' )
	{
		$cmdaudio = ' -oac copy ';
	}
	elsif( getSetting( 'audiocodec' ) eq 'pcm' )
	{
		$cmdaudio = ' -oac pcm ';
	}

	## make vop options
	my $cmdvop = ' -vop '.getSetting( 'filters' ).',scale='.getSetting( 'scalew' ).':'.getSetting( 'scaleh' ).' ';

	## make output FPS options
	my $cmdofps = '';
	if( getSetting( 'fpsset' ) )
	{
		my $ofps = getSetting( 'ofps' );
		$ofps  =~ s/\,/\./g; ## locales decimal
		$cmdofps = ' -ofps '.$ofps.' ';
	}

	## make codec options
	my $videobitrate = $widgets->{'entryVideoBitrate'}->get_text;
	my $cmdcodec = '';
	if( getSetting( 'videocodec' ) eq 'divx4' )
	{
		$cmdcodec = ' -ovc divx4 -divx4opts '.getSetting( 'divx4opts' ).':br='.$videobitrate.':pass';
	}
	elsif( getSetting( 'videocodec' ) eq 'lavc' )
	{
		$cmdcodec = ' -ovc lavc -lavcopts '.getSetting( 'lavcopts' ).':vbitrate='.$videobitrate.':vpass';
	}
	elsif( getSetting( 'videocodec' ) eq 'copy' )
	{
		$cmdcodec = ' -ovc copy ';
	}


	## make subtitle + audio id options
	my $cmdsubaudioids = '';
	if( getSetting( 'subtype' ) eq 'from SID' )
	{
		$cmdsubaudioids .= ' -sid '.getSetting( 'sid' ).' ';
	}
	if( $widgets->{'chkbtnAudioAID'}->get_active )
	{
		$cmdsubaudioids .= ' -aid '.getSetting( 'aid' ).' ';
	}

	## make subtitle file options
	my $cmdsubfile = '';
	if( getSetting( 'subtype' ) eq 'from file' )
	{
		$cmdsubfile = ' -sub "'.$widgets->{'entrySubtitleFromFile'}->get_text.'" ';
	}

	## make video [from,to] options
	my $cmdvideofromto = '';
	if( $widgets->{'chkbtnVideoStart'}->get_active )
	{
		$cmdvideofromto .= ' -ss '.$widgets->{'entryVideoStart'}->get_text.' ';
	}
	if( $widgets->{'chkbtnVideoEnd'}->get_active )
	{
		$cmdvideofromto .= ' -endpos '.$widgets->{'entryVideoEnd'}->get_text.'mb ';
	}


	## mencoder path
	my $mencoder = getSetting( 'mencoderpath' );

	## make all the commands
	## is an array -> 0-2 for 1, 2 or 3 pass encoding
	my @cmds = ();
	if( getSetting( 'videocodec' ) ne 'copy' )
	{
		if( getSetting( 'passes' ) == 1 )
		{
			$cmds[0] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdofps $cmdvop $cmdaudio $cmdcodec=1 $cmdsubfile -o \"$outputfile\" \"$inputfile\" ";
		}
		elsif( getSetting( 'passes' ) == 2 )
		{
			$cmds[0] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdofps $cmdvop $cmdaudio $cmdcodec=1 $cmdsubfile -o /dev/null \"$inputfile\" ";
			$cmds[1] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdofps $cmdvop $cmdaudio $cmdcodec=2 $cmdsubfile -o \"$outputfile\" \"$inputfile\" ";
		}
		elsif( getSetting( 'passes' ) == 3 )
		{
			$cmds[0] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdofps -ovc frameno $cmdaudio $cmdsubfile -o frameno.avi \"$inputfile\" ";
			$cmds[1] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdofps $cmdvop -oac copy $cmdcodec=1 $cmdsubfile -o /dev/null \"$inputfile\" ";
			$cmds[2] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdofps $cmdvop -oac copy $cmdcodec=2 $cmdsubfile -o \"$outputfile\" \"$inputfile\" ";
		}
	}
	else
	{
		@cmds = ();
		$cmds[0] = "$mencoder $cmdsubaudioids $cmdvideofromto $cmdaudio $cmdcodec -o \"$outputfile\" \"$inputfile\" ";
	}

	## clean up before?
	if( 	getSetting( 'cleanup' ) eq 'before' ||
		getSetting( 'cleanup' ) eq 'before+after' )
	{ cleanUp(); }

	my $encodingresult = TRUE;
	for( my $i=0; $i<@cmds; ++$i )
	{
		if( getSetting( 'isencodingstatus' ) )
		{
			if( !encodePass( $cmds[$i], $i+1, $widgets ) )
			{
				setError( getError().' while encoding pass '.($i+1) );
				$encodingresult = FALSE;
				return FALSE;
			}
		}
		else
		{
			setError( 'Encoding stopped' );
			return FALSE;
		}
	}

	## only clean up after encoding is encoding was successful
	## clean up after?
	if(	$encodingresult &&
		(	getSetting( 'cleanup' ) eq 'after' ||
			getSetting( 'cleanup' ) eq 'before+after'
		)										)
	{ cleanUp(); }

	return $encodingresult;
}

## clean up temporary files
sub cleanUp
{
	unlink "frameno.avi", "divx2pass.log", "analyse.log", "c:\\trace_b.txt";
}

## encode the pass with command attached
sub encodePass
{
	my ($cmd,$pass,$widgets) = @_;
	printMsg( $widgets, $cmd );

	## set label and reset progress bar
	printMsg( $widgets,  'Encoding pass '.$pass );
	$widgets->{pbarBar}->set_text( '0 %' );
	$widgets->{pbarBar}->set_fraction( 0 );
	while( Gtk2->events_pending() ) { Gtk2->main_iteration() };

	## get spec fps and seconds
	## if the time in entryVideoLength != spec length, the time of entryVideoLength is used
	my $specfps = getFileSpec( 'fps' );
	my $specsec = getFileSpec( 'length' );
	if( HtoS( $widgets->{'entryVideoLength'}->get_text ) ne $specsec )
	{ $specsec = HtoS( $widgets->{'entryVideoLength'}->get_text ); }

	## execute command
	my $res = open( TMP, "$cmd 2>&1 |" );
	if( !defined($res) )
	{
		setError( 'Could not encode pass '.$pass );
		return FALSE;
	}
	## remember the mencoder pid -> for stopping the process
	setSetting( 'mencoderpid', $res );

	$/="\r"; ## from http://acidrip.thirtythreeandathird.net/
	## loop and parse the output of mencoder
	while( <TMP> )
	{
		#Pos:  49.0s   1179f ( 1%) 232fps Trem:   8min  69mb  A-V:0.084 [0:115]
		#Pos:3439,7s  82468f (63%) 2401fps Trem:  0min 694mb  A-V:0,043 [929:135]
		if( $_ =~ /Pos:\s{0,2}(.+)s\s+(\d+)f\s\(\s?(\d+)%\)\s*(\d+)fps\sTrem:\s+(\d+)min\s+(\d+)mb\s+A-V:(\d+\.\d+)\s\[(\d+):(\d+)\]/  ||
		    $_ =~ /Pos:\s{0,2}(.+)s\s+(\d+)f\s\(\s?(\d+)%\)\s*(\d+)fps\sTrem:\s+(\d+)min\s+(\d+)mb\s+A-V:(\d+\,\d+)\s\[(\d+):(\d+)\]/ )
		{
			my $numframes = $2;
			my $pct = $3;
			my $fps = $4;
			$fps  =~ s/\,/\./g; ## locales decimal
			#my $eta = $5;
			#my $efs = $6; # estimated file size
			my $todotime = StoH( $specsec - ($numframes/$specfps) );
			$todotime  =~ s/\,/\./g; ## locales decimal
			my $donetime = StoH( $numframes / $specfps );
			$donetime  =~ s/\,/\./g; ## locales decimal

			my $str = sprintf( "Encoding pass %d - fps: %4d - done: %s - todo: %s", $pass, $fps, $donetime, $todotime );

			## update the label or the progress bar if necessary
			if ( $widgets->{'pbarBar'}->get_text ne $pct.' %' )
			{
				$widgets->{'pbarBar'}->set_text( $pct.' %' );
				$widgets->{'pbarBar'}->set_fraction( $pct/100 );
			}
			if( $widgets->{'lblDebug'}->get_text ne $str )
			{
				$widgets->{'lblDebug'}->set_text( $str );
			}
			while( Gtk2->events_pending() ) { Gtk2->main_iteration() };
		}
		while( Gtk2->events_pending() ) { Gtk2->main_iteration() };
	}
	$/="\n"; ## from http://acidrip.thirtythreeandathird.net/
	## close
	close( TMP );

	## set progress bar to 100% only if no encoding is stopped
	## if stopped the progress bar stops indicating where it should
	if( getSetting( 'isencodingstatus' ) )
	{
		$widgets->{pbarBar}->set_text( '100 %' );
		$widgets->{pbarBar}->set_fraction( 1 );
	}

	## reset the mencoder pid
	setSetting( 'mencoderpid', -1 );
	return TRUE;
}


1;
