#!/usr/bin/perl -w

##  Copyright 2004 Rien Croonenborghs
##
##    This file is part of file2divx3pass.
##    file2divx4pass is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##    file2divx3pass is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    You should have received a copy of the GNU General Public License
##    along with file2divx3pass; if not, write to the Free Software
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package File2DiVX3Pass2;

use strict;
use Gtk2;
use Gtk2::SimpleMenu;
use Gtk2::SimpleList;
use constant TRUE  => 1;
use constant FALSE => 0;
use constant VERSION => "2.1";
use File2DiVX3Pass2::Signals;
use File2DiVX3Pass2::Helpers;

sub new
{
	my $class = shift;
	my $widgets;
	my $menu_items =
	[
		Project => 	{
					item_type => '<Branch>',
					children => 	[
									_New => 	{
												item_type => '<StockItem>',
												callback => sub{ menuNewProject( $widgets ) },
												callback_action => 0,
												accelerator => '<ctrl>N',
												extra_data => 'gtk-new',
											},
									_Open =>	{
												item_type => '<StockItem>',
												callback => sub{ menuOpenProject( $widgets ) },
												callback_action => 1,
												accelerator => '<ctrl>O',
												extra_data => 'gtk-open',
											},
									_Save =>	{
												item_type => '<StockItem>',
												callback => sub{ menuSaveProject( $widgets ) },
												callback_action => 2,
												accelerator => '<ctrl>S',
												extra_data => 'gtk-save-as',
											},
									Separator => { item_type => '<Separator>', },
									_Batch =>	{
												item_type => '<StockItem>',
												callback => sub{ menuBatchEndcoding( $widgets ) },
												#callback => sub{ printError( $widgets, 'not done yet' ); },
												callback_action => 3,
												accelerator => '<ctrl>B',
												extra_data => 'gtk-execute',
											},
									Separator => { item_type => '<Separator>', },
									_Quit =>	{
												item_type => '<StockItem>',
												callback => sub {
													print 'Thank you for using File2DiVX3Pass '.VERSION."\n";
													Gtk2->main_quit; },
												callback_action => 4,
												accelerator => '<ctrl>Q',
												extra_data => 'gtk-quit',
											},
								]
				},
		Settings =>	{
						item_type => '<Branch>',
						children =>	[
										Loa_d =>	{
													item_type => '<StockItem>',
													callback => sub{ menuLoadSettings($widgets); },
													callback_action => 5,
             											accelerator => '<ctrl>D',
													extra_data => 'gtk-revert-to-saved',
												},
										Sav_e =>	{
													item_type => '<StockItem>',
													callback => sub{ menuSaveSettings($widgets); },
													callback_action => 6,
													accelerator => '<ctrl>E',
													extra_data => 'gtk-save',
												},
									]
					},
#		A_bout => 	{
#						callbackaction => 9,
#					}
	];
	$widgets->{'factory'} = Gtk2::IconFactory->new;
	$widgets->{'factory'}->add_default;
	$widgets->{'style'} = new Gtk2::Style;
	my @stockitems = (
		{
			stock_id => 'f2d3p-startencoding',
			label => 'Start Encoding',
			icon => 'gtk-yes'
		},
		{
			stock_id => 'f2d3p-stopencoding',
			label => 'Stop Encoding',
			icon => 'gtk-no'
		},
		{
			stock_id => 'f2d3p-setinputfile',
			label => 'Set ',
			icon => 'gtk-open'
		},
		{
			stock_id => 'f2d3p-setsubtitlefile',
			label => 'Set ',
			icon => 'gtk-open'
		},
		{
			stock_id => 'f2d3p-debugclosebutton',
			label => 'close window ',
			icon => 'gtk-quit'
		},
		{
			stock_id => 'f2d3p-batchaddproject',
			label => 'add project(s) ',
			icon => 'gtk-add'
		},
		{
			stock_id => 'f2d3p-batchremoveproject',
			label => 'remove project(s) ',
			icon => 'gtk-remove'
		},
		{
			stock_id => 'f2d3p-batchclearprojects',
			label => 'clear projects ',
			icon => 'gtk-cancel'
		},
		{
			stock_id => 'f2d3p-batchstart',
			label => 'Start ',
			icon => 'gtk-yes'
		},
		{
			stock_id => 'f2d3p-batchstop',
			label => 'Stop ',
			icon => 'gtk-no'
		},
	);
	Gtk2::Stock->add( @stockitems );
	foreach my $stockitem( @stockitems )
	{
		$widgets->{'factory'}->add(	$stockitem->{'stock_id'},
					$widgets->{'style'}->lookup_icon_set( $stockitem->{'icon'} )
		);
        }



	## main window
	$widgets->{'wdwMain'} = Gtk2::Window->new;
	$widgets->{'wdwMain'}->set_title ( 'File2DiVX3Pass '.VERSION );
	$widgets->{'wdwMain'}->set_border_width( 3 );
	$widgets->{'wdwMain'}->set_default_size( 400, 480 );
	$widgets->{'wdwMain'}->set_resizable( FALSE );

	## main vbox, top=1.menu, bottom=2.notebook + 3.start/stop + 4.debug + 5.progress
	$widgets->{'vbMain'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'wdwMain'}->add( $widgets->{'vbMain'} );

	## 1. menu
	$widgets->{'mbarMenu'} = Gtk2::SimpleMenu->new(	menu_tree => $menu_items );
	$widgets->{'wdwMain'}->add_accel_group( $widgets->{'mbarMenu'}->{accel_group} );
	$widgets->{'vbMain'}->pack_start( $widgets->{'mbarMenu'}->{widget}, FALSE, TRUE, 1 );

	##
	my $titlepng = '';
	foreach my $INC( @INC )
	{
		if( -f $INC.'/File2DiVX3Pass2/title.png' )
		{
			$titlepng = $INC.'/File2DiVX3Pass2/title.png';
		}
	}
	$widgets->{'imgTitle'} = Gtk2::Image->new_from_file( $titlepng );
	$widgets->{'vbMain'}->pack_start( $widgets->{'imgTitle'}, FALSE, TRUE, 1 );

	## 2. notebook 21: IO, 22: SUBTITLES, 23: AUDIO, 24: VIDEO, 25: PROGRAMS+SETTINGS
	$widgets->{'nbMain'} = Gtk2::Notebook->new;
	$widgets->{'vbMain'}->pack_start( $widgets->{'nbMain'}, FALSE, TRUE, 1 );


	## 21 IO
#-------------------------------------------------------------------------------
	$widgets->{'vbIO'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'vbIO'}->set_border_width( 3 );
	$widgets->{'nbMain'}->append_page( $widgets->{'vbIO'}, ' Input / Output ' );

		$widgets->{'frmIO'} = Gtk2::Frame->new( ' Input / Output ' );
		$widgets->{'frmIO'}->set_border_width( 1 );
	$widgets->{'vbIO'}->pack_start( $widgets->{'frmIO'}, FALSE, TRUE, 1 );

			$widgets->{'vbIOPage'} = Gtk2::VBox->new( FALSE, 1 );
			$widgets->{'vbIOPage'}->set_border_width( 3 );
		$widgets->{'frmIO'}->add( $widgets->{'vbIOPage'} );

				## file input fields
				$widgets->{'hbIOInput'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbIOInput'}->set_border_width( 1 );
			$widgets->{'vbIOPage'}->pack_start( $widgets->{'hbIOInput'}, FALSE, TRUE, 1 );
					$widgets->{'lblIOInput'} = Gtk2::Label->new( 'Input file' );
					$widgets->{'entryIOInput'} = Gtk2::Entry->new;
					$widgets->{'btnIOInput'} = Gtk2::Button->new_from_stock( 'f2d3p-setinputfile' );
				$widgets->{'hbIOInput'}->pack_start( $widgets->{'lblIOInput'}, FALSE, TRUE, 1 );
				$widgets->{'hbIOInput'}->pack_start( $widgets->{'entryIOInput'}, TRUE, TRUE, 1 );
				$widgets->{'hbIOInput'}->pack_start( $widgets->{'btnIOInput'}, FALSE, TRUE, 1 );
				## file output fields
				$widgets->{'hbIOOutput'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbIOOutput'}->set_border_width( 1 );
			$widgets->{'vbIOPage'}->pack_start( $widgets->{'hbIOOutput'}, FALSE, TRUE, 1 );
					$widgets->{'lblIOOutput'} = Gtk2::Label->new( 'Output file' );
					$widgets->{'entryIOOutput'} = Gtk2::Entry->new;
					$widgets->{'entryIOOutput'}->set_text( 'output.divx.avi' );
				$widgets->{'hbIOOutput'}->pack_start( $widgets->{'lblIOOutput'}, FALSE, TRUE, 1 );
				$widgets->{'hbIOOutput'}->pack_start( $widgets->{'entryIOOutput'}, TRUE, TRUE, 1 );


	## 22 SUBTITLES
#-------------------------------------------------------------------------------
	$widgets->{'vbSubtitles'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'vbSubtitles'}->set_border_width( 3 );
	$widgets->{'nbMain'}->append_page( $widgets->{'vbSubtitles'}, ' Subtitles ' );

		$widgets->{'frmSubtitles'} = Gtk2::Frame->new( ' Subtitles ' );
		$widgets->{'frmSubtitles'}->set_border_width( 1 );
	$widgets->{'vbSubtitles'}->pack_start( $widgets->{'frmSubtitles'}, FALSE, TRUE, 3 );

			$widgets->{'vbSubtitlesPage'} = Gtk2::VBox->new( FALSE, 1 );
			$widgets->{'vbSubtitlesPage'}->set_border_width( 3 );
		$widgets->{'frmSubtitles'}->add( $widgets->{'vbSubtitlesPage'} );

				## subtitle menu
				$widgets->{'hbSubtitleType'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbSubtitleType'}->set_border_width( 1 );
			$widgets->{'vbSubtitlesPage'}->pack_start( $widgets->{'hbSubtitleType'}, TRUE, TRUE, 1 );
					$widgets->{'lblSubtitleType'} = Gtk2::Label->new( 'Use subtitle' );
					$widgets->{'optmenSubtitleType'} = Gtk2::OptionMenu->new;
					$widgets->{'menSubtitleType'} = Gtk2::Menu->new;
					$widgets->{'optmenSubtitleType'}->set_menu( $widgets->{'menSubtitleType'} );
					foreach my $suboption( 'no', 'from file', 'from SID' )
					{
						$widgets->{'menitSubtitleType'} = Gtk2::MenuItem->new( $suboption );
						$widgets->{'menitSubtitleType'}->signal_connect( activate => \&menitSubtitleTypeActivated, $widgets );
						$widgets->{'menSubtitleType'}->append( $widgets->{'menitSubtitleType'} );
					}
					$widgets->{'optmenSubtitleType'}->set_history( 0 );
					$widgets->{'optmenSubtitleType'}->set_size_request( 100, -1);
				$widgets->{'hbSubtitleType'}->pack_start( $widgets->{'lblSubtitleType'}, FALSE, TRUE, 1 );
				$widgets->{'hbSubtitleType'}->pack_start( $widgets->{'optmenSubtitleType'}, FALSE, TRUE, 1 );
				## subtitle from file fields
				$widgets->{'hbSubtitleFromFile'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbSubtitleFromFile'}->set_border_width( 1 );
			$widgets->{'vbSubtitlesPage'}->pack_start( $widgets->{'hbSubtitleFromFile'}, TRUE, TRUE, 1 );
					$widgets->{'lblSubtitleFromFile'} = Gtk2::Label->new( 'Subtitle file' );
					$widgets->{'lblSubtitleFromFile'}->set_sensitive( FALSE );
					$widgets->{'entrySubtitleFromFile'} = Gtk2::Entry->new;
					$widgets->{'entrySubtitleFromFile'}->set_sensitive( FALSE );
					$widgets->{'btnSubtitleFromFile'} = Gtk2::Button->new_from_stock( 'f2d3p-setsubtitlefile' );
					$widgets->{'btnSubtitleFromFile'}->sensitive( FALSE );
				$widgets->{'hbSubtitleFromFile'}->pack_start( $widgets->{'lblSubtitleFromFile'}, FALSE, TRUE, 1 );
				$widgets->{'hbSubtitleFromFile'}->pack_start( $widgets->{'entrySubtitleFromFile'}, TRUE, TRUE, 1 );
				$widgets->{'hbSubtitleFromFile'}->pack_start( $widgets->{'btnSubtitleFromFile'}, FALSE, TRUE, 1 );
				## subtitle from SID fields
				$widgets->{'hbSubtitleFromSID'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbSubtitleFromSID'}->set_border_width( 1 );
			$widgets->{'vbSubtitlesPage'}->pack_start( $widgets->{'hbSubtitleFromSID'}, TRUE, TRUE, 1 );
					$widgets->{'lblSubtitleFromSID'} = Gtk2::Label->new( 'SID' );
					$widgets->{'lblSubtitleFromSID'}->set_sensitive( FALSE );
					$widgets->{'adjSubtitleFromSID'} = new Gtk2::Adjustment(1, 1, 31, 1, 1, 1);
                		$widgets->{'spinSubtitleFromSID'} = new Gtk2::SpinButton( $widgets->{'adjSubtitleFromSID'}, 1, 0 );
                		$widgets->{'spinSubtitleFromSID'}->set_size_request(40, -1);
					$widgets->{'spinSubtitleFromSID'}->set_sensitive( FALSE );
				$widgets->{'hbSubtitleFromSID'}->pack_start( $widgets->{'lblSubtitleFromSID'}, FALSE, TRUE, 1 );
				$widgets->{'hbSubtitleFromSID'}->pack_start( $widgets->{'spinSubtitleFromSID'}, FALSE, TRUE, 1 );

	## 23 AUDIO
#-------------------------------------------------------------------------------
	$widgets->{'vbAudio'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'vbAudio'}->set_border_width( 3 );
	$widgets->{'nbMain'}->append_page( $widgets->{'vbAudio'}, ' Audio ' );

		$widgets->{'frmAudio'} = Gtk2::Frame->new( ' Audio ' );
		$widgets->{'frmAudio'}->set_border_width( 1 );
	$widgets->{'vbAudio'}->pack_start( $widgets->{'frmAudio'}, FALSE, TRUE, 3 );

			$widgets->{'vbAudioPage'} = Gtk2::VBox->new( FALSE, 1 );
			$widgets->{'vbAudioPage'}->set_border_width( 3 );
		$widgets->{'frmAudio'}->add( $widgets->{'vbAudioPage'} );

				## audio codec menu
				$widgets->{'hbAudioCodec'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbAudioCodec'}->set_border_width( 1 );
			$widgets->{'vbAudioPage'}->pack_start( $widgets->{'hbAudioCodec'}, TRUE, TRUE, 1 );
					$widgets->{'lblAudioCodec'} = Gtk2::Label->new( 'Audio Codec' );
					$widgets->{'optmenAudioCodec'} = Gtk2::OptionMenu->new;
					$widgets->{'menAudioCodec'} = Gtk2::Menu->new;
					$widgets->{'optmenAudioCodec'}->set_menu( $widgets->{'menAudioCodec'} );
					foreach my $audiocodec( 'MP3', 'copy', 'PCM' )
					{
						$widgets->{'menitAudioCodec'} = Gtk2::MenuItem->new( $audiocodec );
						$widgets->{'menitAudioCodec'}->signal_connect( activate => \&menitAudioCodecActivated, $widgets );
						$widgets->{'menAudioCodec'}->append( $widgets->{'menitAudioCodec'} );
					}
					$widgets->{'optmenAudioCodec'}->set_history( 0 );
					$widgets->{'optmenAudioCodec'}->set_size_request( 75, -1);
				$widgets->{'hbAudioCodec'}->pack_start( $widgets->{'lblAudioCodec'}, FALSE, TRUE, 1 );
				$widgets->{'hbAudioCodec'}->pack_start( $widgets->{'optmenAudioCodec'}, FALSE, TRUE, 1 );
				## MP3 codec fields
				$widgets->{'hbAudioMP3'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbAudioMP3'}->set_border_width( 1 );
			$widgets->{'vbAudioPage'}->pack_start( $widgets->{'hbAudioMP3'}, TRUE, TRUE, 1 );
					$widgets->{'lblAudioMP3'} = Gtk2::Label->new( 'MP3 ' );
					$widgets->{'optmenAudioMP3Rate'} = Gtk2::OptionMenu->new;
					$widgets->{'menAudioMP3Rate'} = Gtk2::Menu->new;
					$widgets->{'optmenAudioMP3Rate'}->set_menu( $widgets->{'menAudioMP3Rate'} );
					foreach my $rate( 'constant', 'average' )
					{
						$widgets->{'menitAudioMP3Rate'} = Gtk2::MenuItem->new( $rate );
						$widgets->{'menitAudioMP3Rate'}->signal_connect( activate => \&menitAudioMP3RateActivated, $widgets );
						$widgets->{'menAudioMP3Rate'}->append( $widgets->{'menitAudioMP3Rate'} );
					}
					$widgets->{'optmenAudioMP3Rate'}->set_history( 0 );
					$widgets->{'optmenAudioMP3Rate'}->set_size_request( 95, -1);
					$widgets->{'lblAudioMP3at'} = Gtk2::Label->new( 'bitrate at' );
					$widgets->{'optmenAudioMP3Bitrate'} = Gtk2::OptionMenu->new;
					$widgets->{'menAudioMP3Bitrate'} = Gtk2::Menu->new;
					$widgets->{'optmenAudioMP3Bitrate'}->set_menu( $widgets->{'menAudioMP3Bitrate'} );
					foreach my $mp3bitrate( getMP3Bitrates() )
					{
						$widgets->{'menitAudioMP3Bitrate'} = Gtk2::MenuItem->new( $mp3bitrate );
						$widgets->{'menitAudioMP3Bitrate'}->signal_connect( activate => \&menitAudioMP3BitrateActivated, $widgets );
						$widgets->{'menAudioMP3Bitrate'}->append( $widgets->{'menitAudioMP3Bitrate'} );
					}
					$widgets->{'optmenAudioMP3Bitrate'}->set_history( 8 );
					$widgets->{'optmenAudioMP3Bitrate'}->set_size_request( 65, -1);
					$widgets->{'lblAudioMP3kps'} = Gtk2::Label->new( 'kb/s' );
				$widgets->{'hbAudioMP3'}->pack_start( $widgets->{'lblAudioMP3'}, FALSE, TRUE, 1 );
				$widgets->{'hbAudioMP3'}->pack_start( $widgets->{'optmenAudioMP3Rate'}, FALSE, TRUE, 1 );
				$widgets->{'hbAudioMP3'}->pack_start( $widgets->{'lblAudioMP3at'}, FALSE, TRUE, 1 );
				$widgets->{'hbAudioMP3'}->pack_start( $widgets->{'optmenAudioMP3Bitrate'}, FALSE, TRUE, 1 );
				$widgets->{'hbAudioMP3'}->pack_start( $widgets->{'lblAudioMP3kps'}, FALSE, TRUE, 1 );
				## AID fields
				$widgets->{'hbAudioAID'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbAudioAID'}->set_border_width( 1 );
			$widgets->{'vbAudioPage'}->pack_start( $widgets->{'hbAudioAID'}, TRUE, TRUE, 1 );
					$widgets->{'chkbtnAudioAID'} = Gtk2::CheckButton->new( 'AID' );
					$widgets->{'adjAudioAID'} = new Gtk2::Adjustment(1, 1, 31, 1, 1, 1);
                		$widgets->{'spinAudioAID'} = new Gtk2::SpinButton( $widgets->{'adjAudioAID'}, 1, 0 );
                		$widgets->{'spinAudioAID'}->set_size_request(40, -1);
					$widgets->{'spinAudioAID'}->set_sensitive( FALSE );
				$widgets->{'hbAudioAID'}->pack_start( $widgets->{'chkbtnAudioAID'}, FALSE, TRUE, 1 );
				$widgets->{'hbAudioAID'}->pack_start( $widgets->{'spinAudioAID'}, FALSE, TRUE, 1 );


	## 24 VIDEO
#-------------------------------------------------------------------------------
	$widgets->{'vbVideo'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'vbVideo'}->set_border_width( 3 );
	$widgets->{'nbMain'}->append_page( $widgets->{'vbVideo'}, ' Video ' );

		$widgets->{'frmVideo'} = Gtk2::Frame->new( ' Video ' );
		$widgets->{'frmVideo'}->set_border_width( 1 );
	$widgets->{'vbVideo'}->pack_start( $widgets->{'frmVideo'}, FALSE, TRUE, 3 );

			$widgets->{'vbVideoPage'} = Gtk2::VBox->new( FALSE, 1 );
			$widgets->{'vbVideoPage'}->set_border_width( 3 );
		$widgets->{'frmVideo'}->add( $widgets->{'vbVideoPage'} );

				$widgets->{'tableVideo'} = Gtk2::Table->new( 10, 2, FALSE );
			$widgets->{'vbVideoPage'}->pack_start( $widgets->{'tableVideo'}, TRUE, TRUE, 1 );

					## video length fields
					$widgets->{'hbVideoLength'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoLength'}->set_border_width( 3 );
						$widgets->{'lblVideoLength'} = Gtk2::Label->new( 'Video Length' );
						$widgets->{'entryVideoLength'} = Gtk2::Entry->new;
						$widgets->{'entryVideoLength'}->set_text( '00:00:00' );
						$widgets->{'entryVideoLength'}->set_size_request(65, -1);
					$widgets->{'hbVideoLength'}->pack_start( $widgets->{'lblVideoLength'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoLength'}->pack_start( $widgets->{'entryVideoLength'}, FALSE, TRUE, 1 );
					## video start
					$widgets->{'hbVideoStart'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoStart'}->set_border_width( 3 );
						$widgets->{'chkbtnVideoStart'} = Gtk2::CheckButton->new( 'Video start at' );
						$widgets->{'entryVideoStart'} = Gtk2::Entry->new;
						$widgets->{'entryVideoStart'}->set_text( '00:00:00' );
						$widgets->{'entryVideoStart'}->set_size_request(65, -1);
					$widgets->{'hbVideoStart'}->pack_start( $widgets->{'chkbtnVideoStart'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoStart'}->pack_start( $widgets->{'entryVideoStart'}, FALSE, TRUE, 1 );
					## video End
					$widgets->{'hbVideoEnd'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoEnd'}->set_border_width( 3 );
						$widgets->{'chkbtnVideoEnd'} = Gtk2::CheckButton->new( 'end at' );
						$widgets->{'entryVideoEnd'} = Gtk2::Entry->new;
						$widgets->{'entryVideoEnd'}->set_text( '700' );
						$widgets->{'entryVideoEnd'}->set_size_request(50, -1);
						$widgets->{'lblVideoEnd'} = Gtk2::Label->new( 'Mb' );
					$widgets->{'hbVideoEnd'}->pack_start( $widgets->{'chkbtnVideoEnd'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoEnd'}->pack_start( $widgets->{'entryVideoEnd'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoEnd'}->pack_start( $widgets->{'lblVideoEnd'}, FALSE, TRUE, 1 );
					## cd size fields
					$widgets->{'hbVideoSize'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoSize'}->set_border_width( 3 );
						$widgets->{'lblVideoCDSize'} = Gtk2::Label->new( 'CD Size' );
						$widgets->{'entryVideoCDSize'} = Gtk2::Entry->new;
						$widgets->{'entryVideoCDSize'}->set_text( getSetting( 'cdsize' ) );
						$widgets->{'entryVideoCDSize'}->set_size_request(50, -1);
						$widgets->{'lblVideoCDSizeMb'} = Gtk2::Label->new( 'Mb' );
					$widgets->{'hbVideoSize'}->pack_start( $widgets->{'lblVideoCDSize'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoSize'}->pack_start( $widgets->{'entryVideoCDSize'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoSize'}->pack_start( $widgets->{'lblVideoCDSizeMb'}, FALSE, TRUE, 1 );
					## output FPS fields
					$widgets->{'hbVideoFPS'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoFPS'}->set_border_width( 3 );
						$widgets->{'chkbtnVideoOutputFPS'} = Gtk2::CheckButton->new( 'Output FPS' );
						$widgets->{'entryVideoOutputFPS'} = Gtk2::Entry->new;
						$widgets->{'entryVideoOutputFPS'}->set_text( getSetting( 'ofps' ) );
						$widgets->{'entryVideoOutputFPS'}->set_size_request(55, -1);
						$widgets->{'entryVideoOutputFPS'}->set_sensitive( FALSE );
					$widgets->{'hbVideoFPS'}->pack_start( $widgets->{'chkbtnVideoOutputFPS'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoFPS'}->pack_start( $widgets->{'entryVideoOutputFPS'}, FALSE, TRUE, 1 );
					## scale WxH fields
					$widgets->{'hbVideoScale'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoScale'}->set_border_width( 3 );
						$widgets->{'lblVideoScale1'} = Gtk2::Label->new( 'Scale W' );
						$widgets->{'entryVideoScaleW'} = Gtk2::Entry->new;
						$widgets->{'entryVideoScaleW'}->set_text( getSetting( 'scalew' ) );
						$widgets->{'entryVideoScaleW'}->set_size_request(40, -1);
						$widgets->{'lblVideoScale2'} = Gtk2::Label->new( 'x H' );
						$widgets->{'entryVideoScaleH'} = Gtk2::Entry->new;
						$widgets->{'entryVideoScaleH'}->set_text( getSetting( 'scaleh' ) );
						$widgets->{'entryVideoScaleH'}->set_size_request(40, -1);
					$widgets->{'hbVideoScale'}->pack_start( $widgets->{'lblVideoScale1'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoScale'}->pack_start( $widgets->{'entryVideoScaleW'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoScale'}->pack_start( $widgets->{'lblVideoScale2'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoScale'}->pack_start( $widgets->{'entryVideoScaleH'}, FALSE, TRUE, 1 );
					## scale ratio fields
					$widgets->{'hbVideoRatio'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoRatio'}->set_border_width( 3 );
						$widgets->{'rbtnVideoRation43'} = Gtk2::RadioButton->new_from_widget( undef, '4:3' );
						$widgets->{'rbtnVideoRation169'} = Gtk2::RadioButton->new_from_widget( $widgets->{'rbtnVideoRation43'}, '16:9' );
					$widgets->{'hbVideoRatio'}->pack_start( $widgets->{'rbtnVideoRation43'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoRatio'}->pack_start( $widgets->{'rbtnVideoRation169'}, FALSE, TRUE, 1 );
					## video bitrate fields
					$widgets->{'hbVideoBitrate'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoBitrate'}->set_border_width( 3 );
						$widgets->{'lblVideoBitrate'} = Gtk2::Label->new( 'Video Bitrate' );
						$widgets->{'entryVideoBitrate'} = Gtk2::Entry->new;
						$widgets->{'entryVideoBitrate'}->set_text( '0' );
						$widgets->{'entryVideoBitrate'}->set_size_request(50, -1);
					$widgets->{'hbVideoBitrate'}->pack_start( $widgets->{'lblVideoBitrate'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoBitrate'}->pack_start( $widgets->{'entryVideoBitrate'}, FALSE, TRUE, 1 );
					## video BPP fields
					$widgets->{'hbVideoBPP'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoBPP'}->set_border_width( 3 );
						$widgets->{'lblVideoBPP'} = Gtk2::Label->new( 'Bits/Pixel' );
						$widgets->{'entryVideoBPP'} = Gtk2::Entry->new;
						$widgets->{'entryVideoBPP'}->set_text( '0' );
						$widgets->{'entryVideoBPP'}->set_size_request(50, -1);
					$widgets->{'hbVideoBPP'}->pack_start( $widgets->{'lblVideoBPP'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoBPP'}->pack_start( $widgets->{'entryVideoBPP'}, FALSE, TRUE, 1 );
					## video codec fields
					$widgets->{'hbVideoCodec'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoCodec'}->set_border_width( 3 );
						$widgets->{'lblVideoCodec'} = Gtk2::Label->new( 'Video Codec' );
						$widgets->{'optmenVideoCodec'} = Gtk2::OptionMenu->new;
						$widgets->{'menVideoCodec'} = Gtk2::Menu->new;
						$widgets->{'optmenVideoCodec'}->set_menu( $widgets->{'menVideoCodec'} );
						foreach my $videocodec( 'lavc', 'divx4', 'copy' )
						{
							$widgets->{'menitVideoCodec'} = Gtk2::MenuItem->new( $videocodec );
							$widgets->{'menitVideoCodec'}->signal_connect( activate => \&menitVideoCodecActivated, $widgets );
							$widgets->{'menVideoCodec'}->append( $widgets->{'menitVideoCodec'} );
						}
						$widgets->{'optmenVideoCodec'}->set_history( 0 );
						$widgets->{'optmenVideoCodec'}->set_size_request( 75, -1);
					$widgets->{'hbVideoCodec'}->pack_start( $widgets->{'lblVideoCodec'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoCodec'}->pack_start( $widgets->{'optmenVideoCodec'}, FALSE, TRUE, 1 );
					## video pass fields
					$widgets->{'hbVideoPass'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoPass'}->set_border_width( 3 );
						$widgets->{'lblVideoPass'} = Gtk2::Label->new( 'Passes' );
						$widgets->{'optmenVideoPass'} = Gtk2::OptionMenu->new;
						$widgets->{'menVideoPass'} = Gtk2::Menu->new;
						$widgets->{'optmenVideoPass'}->set_menu( $widgets->{'menVideoPass'} );
						foreach my $pass( 1, 2, 3 )
						{
							$widgets->{'menitVideoPass'} = Gtk2::MenuItem->new( $pass );
							$widgets->{'menitVideoPass'}->signal_connect( activate => \&menitVideoPassActivated, $widgets );
							$widgets->{'menVideoPass'}->append( $widgets->{'menitVideoPass'} );
						}
						$widgets->{'optmenVideoPass'}->set_history( 0 );
						$widgets->{'optmenVideoPass'}->set_size_request( 50, -1);
					$widgets->{'hbVideoPass'}->pack_start( $widgets->{'lblVideoPass'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoPass'}->pack_start( $widgets->{'optmenVideoPass'}, FALSE, TRUE, 1 );
					## encoding options fields
					$widgets->{'hbVideoEncOpts'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoEncOpts'}->set_border_width( 3 );
						$widgets->{'lblVideoEncOpts'} = Gtk2::Label->new( 'Encoding Options' );
						$widgets->{'entryVideoEncOpts'} = Gtk2::Entry->new;
						$widgets->{'entryVideoEncOpts'}->set_text( getSetting( 'lavcopts' ) );
					$widgets->{'hbVideoEncOpts'}->pack_start( $widgets->{'lblVideoEncOpts'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoEncOpts'}->pack_start( $widgets->{'entryVideoEncOpts'}, TRUE, TRUE, 1 );
					## filter options fields
					$widgets->{'hbVideoFilterOpts'} = Gtk2::HBox->new( FALSE, 1 );
					$widgets->{'hbVideoFilterOpts'}->set_border_width( 3 );
						$widgets->{'lblVideoFilterOpts'} = Gtk2::Label->new( 'Filter Options' );
						$widgets->{'entryVideoFilterOpts'} = Gtk2::Entry->new;
						$widgets->{'entryVideoFilterOpts'}->set_text( getSetting( 'filters' ) );
					$widgets->{'hbVideoFilterOpts'}->pack_start( $widgets->{'lblVideoFilterOpts'}, FALSE, TRUE, 1 );
					$widgets->{'hbVideoFilterOpts'}->pack_start( $widgets->{'entryVideoFilterOpts'}, TRUE, TRUE, 1 );


				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoLength'}, 0, 1, 0, 1 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoStart'}, 0, 1, 1, 2 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoEnd'}, 1, 2, 1, 2 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoSize'}, 0, 1, 2, 3 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoFPS'}, 1, 2, 2, 3 );
				$widgets->{'tableVideo'}->attach_defaults( Gtk2::HSeparator->new, 0, 2, 3, 4 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoScale'}, 0, 1, 4, 5 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoRatio'}, 1, 2, 4, 5 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoBitrate'}, 0, 1, 5, 6 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoBPP'}, 1, 2, 5, 6 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoCodec'}, 0, 1, 6, 7 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoPass'}, 1, 2, 6, 7 );
				$widgets->{'tableVideo'}->attach_defaults( Gtk2::HSeparator->new, 0, 2, 7, 8 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoEncOpts'}, 0, 2, 8, 9 );
				$widgets->{'tableVideo'}->attach_defaults( $widgets->{'hbVideoFilterOpts'}, 0, 2, 9, 10 );


	## 25 PROGRAMS + SETTINGS
#-------------------------------------------------------------------------------
$widgets->{'vbProgramsSettings'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'vbProgramsSettings'}->set_border_width( 3 );
	$widgets->{'nbMain'}->append_page( $widgets->{'vbProgramsSettings'}, ' Programs/Settings ' );

		$widgets->{'frmPrograms'} = Gtk2::Frame->new( ' Programs ' );
		$widgets->{'frmPrograms'}->set_border_width( 1 );
	$widgets->{'vbProgramsSettings'}->pack_start( $widgets->{'frmPrograms'}, FALSE, TRUE, 3 );

			$widgets->{'vbPrograms'} = Gtk2::VBox->new( FALSE, 1 );
			$widgets->{'vbPrograms'}->set_border_width( 3 );
		$widgets->{'frmPrograms'}->add( $widgets->{'vbPrograms'} );

				## mplayer program
				$widgets->{'hbProgramsMPlayer'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbProgramsMPlayer'}->set_border_width( 1 );
			$widgets->{'vbPrograms'}->pack_start( $widgets->{'hbProgramsMPlayer'}, TRUE, TRUE, 1 );
					$widgets->{'lblProgramsMPlayer'} = Gtk2::Label->new( 'MPlayer' );
					$widgets->{'entryProgramsMPlayer'} = Gtk2::Entry->new;
					$widgets->{'entryProgramsMPlayer'}->set_text( getSetting( 'mplayerpath' ) );
				$widgets->{'hbProgramsMPlayer'}->pack_start( $widgets->{'lblProgramsMPlayer'}, FALSE, TRUE, 1 );
				$widgets->{'hbProgramsMPlayer'}->pack_start( $widgets->{'entryProgramsMPlayer'}, TRUE, TRUE, 1 );

				## mencoder program
				$widgets->{'hbProgramsMEncoder'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbProgramsMEncoder'}->set_border_width( 1 );
			$widgets->{'vbPrograms'}->pack_start( $widgets->{'hbProgramsMEncoder'}, TRUE, TRUE, 1 );
					$widgets->{'lblProgramsMEncoder'} = Gtk2::Label->new( 'MEncoder' );
					$widgets->{'entryProgramsMEncoder'} = Gtk2::Entry->new;
					$widgets->{'entryProgramsMEncoder'}->set_text( getSetting( 'mencoderpath' ) );
				$widgets->{'hbProgramsMEncoder'}->pack_start( $widgets->{'lblProgramsMEncoder'}, FALSE, TRUE, 1 );
				$widgets->{'hbProgramsMEncoder'}->pack_start( $widgets->{'entryProgramsMEncoder'}, TRUE, TRUE, 1 );

		$widgets->{'frmSettings'} = Gtk2::Frame->new( ' Settings ' );
		$widgets->{'frmSettings'}->set_border_width( 1 );
	$widgets->{'vbProgramsSettings'}->pack_start( $widgets->{'frmSettings'}, FALSE, TRUE, 3 );

			$widgets->{'vbSettings'} = Gtk2::VBox->new( FALSE, 1 );
			$widgets->{'vbSettings'}->set_border_width( 3 );
		$widgets->{'frmSettings'}->add( $widgets->{'vbSettings'} );

				## clean up
				$widgets->{'hbSettingsCleanup'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbSettingsCleanup'}->set_border_width( 1 );
			$widgets->{'vbSettings'}->pack_start( $widgets->{'hbSettingsCleanup'}, TRUE, TRUE, 1 );
					$widgets->{'lblSettingsCleanup'} = Gtk2::Label->new( 'Clean up' );
					$widgets->{'optmenSettingsCleanup'} = Gtk2::OptionMenu->new;
					$widgets->{'menSettingsCleanup'} = Gtk2::Menu->new;
					$widgets->{'optmenSettingsCleanup'}->set_menu ( $widgets->{'menSettingsCleanup'} );
					foreach my $clean( 'before', 'after', 'before+after' )
					{
						$widgets->{'menitSettingsCleanup'} = Gtk2::MenuItem->new( $clean );
						$widgets->{'menitSettingsCleanup'}->signal_connect( activate => \&menitSettingsCleanupActivated, $widgets );
						$widgets->{'menSettingsCleanup'}->append( $widgets->{'menitSettingsCleanup'} );
					}
					$widgets->{'optmenSettingsCleanup'}->set_history( 2 );
					$widgets->{'optmenSettingsCleanup'}->set_size_request( 125, -1);
				$widgets->{'hbSettingsCleanup'}->pack_start( $widgets->{'lblSettingsCleanup'}, FALSE, TRUE, 1 );
				$widgets->{'hbSettingsCleanup'}->pack_start( $widgets->{'optmenSettingsCleanup'}, FALSE, TRUE, 1 );

				## debug
				$widgets->{'hbSettingsDebug'} = Gtk2::HBox->new( FALSE, 1 );
				$widgets->{'hbSettingsDebug'}->set_border_width( 1 );
			$widgets->{'vbSettings'}->pack_start( $widgets->{'hbSettingsDebug'}, TRUE, TRUE, 1 );
					$widgets->{'chkbtnSettingsDebug'} = Gtk2::CheckButton->new( 'show debug window' );
				$widgets->{'hbSettingsDebug'}->pack_start( $widgets->{'chkbtnSettingsDebug'}, FALSE, TRUE, 1 );


	## 5. progress
	$widgets->{'pbarBar'} = Gtk2::ProgressBar->new;
	$widgets->{'vbMain'}->pack_end( $widgets->{'pbarBar'}, FALSE, TRUE, 1 );
		$widgets->{'pbarBar'}->set_orientation('left_to_right');
		$widgets->{'pbarBar'}->set_text( '0 %' );


	## 4. debug
	$widgets->{'hbDebug'} = Gtk2::HBox->new( FALSE, 3 );
	$widgets->{'vbMain'}->pack_end( $widgets->{'hbDebug'}, FALSE, TRUE, 1 );
		$widgets->{'lblDebug'} = Gtk2::Label->new;
		$widgets->{'hbDebug'}->pack_start( $widgets->{'lblDebug'}, FALSE, TRUE, 1 );



	## 3. start/stop
	$widgets->{'btnStopEncoding'} = Gtk2::Button->new_from_stock( 'f2d3p-stopencoding' );
	$widgets->{'btnStopEncoding'}->set_border_width( 0 );
	$widgets->{'vbMain'}->pack_end( $widgets->{'btnStopEncoding'}, FALSE, TRUE, 1 );
	$widgets->{'btnStopEncoding'}->set_sensitive( FALSE );
	$widgets->{'btnStartEncoding'} = Gtk2::Button->new_from_stock( 'f2d3p-startencoding' );
	$widgets->{'btnStartEncoding'}->set_border_width( 0 );
	$widgets->{'vbMain'}->pack_end( $widgets->{'btnStartEncoding'}, FALSE, TRUE, 1 );



	## signals
	$widgets->{'wdwMain'}->signal_connect( destroy => sub { 	print 'Thank you for using File2DiVX3Pass '.VERSION."\n";
													Gtk2->main_quit; } );
	## IO tab
	$widgets->{'btnIOInput'}->signal_connect( clicked => \&btnIOInputClicked, $widgets );
	## SUBTITLE tab
	$widgets->{'adjSubtitleFromSID'}->signal_connect( value_changed => \&adjSubtitleFromSIDValueChanged, $widgets );
	$widgets->{'btnSubtitleFromFile'}->signal_connect( clicked => \&btnSubtitleFromFileClicked, $widgets );
	## AUDIO tab
	$widgets->{'chkbtnAudioAID'}->signal_connect( toggled => \&chkbtnAudioAIDToggled, $widgets );
	$widgets->{'adjAudioAID'}->signal_connect( value_changed => \&adjAudioAIDValueChanged, $widgets );
	## VIDEO tab
	$widgets->{'entryVideoLength'}->signal_connect( activate => \&entryVideoLengthActivated, $widgets );
	$widgets->{'chkbtnVideoStart'}->signal_connect( toggled => \&chkbtnVideoStartToggled, $widgets );
	$widgets->{'entryVideoStart'}->signal_connect( activate => \&entryVideoStartActivated, $widgets );
	$widgets->{'chkbtnVideoEnd'}->signal_connect( toggled => \&chkbtnVideoEndToggled, $widgets );
	$widgets->{'entryVideoEnd'}->signal_connect( activate => \&entryVideoEndActivated, $widgets );
	$widgets->{'entryVideoCDSize'}->signal_connect( activate => \&entryVideoCDSizeActivated, $widgets );
	$widgets->{'chkbtnVideoOutputFPS'}->signal_connect( toggled => \&chkbtnVideoOutputFPSToggled, $widgets );
	$widgets->{'entryVideoOutputFPS'}->signal_connect( activate => \&entryVideoOutputFPSActivated, $widgets );
	$widgets->{'entryVideoScaleW'}->signal_connect( activate => \&entryVideoScaleWActivated, $widgets );
	$widgets->{'entryVideoScaleH'}->signal_connect( activate => \&entryVideoScaleHActivated, $widgets );
	$widgets->{'rbtnVideoRation43'}->signal_connect( toggled => \&rbtnVideoRation43Toggled, $widgets );
	$widgets->{'entryVideoBitrate'}->signal_connect( changed => \&entryVideoBitrateChanged, $widgets );
	## PROGRAM+SETTINGS tab
	$widgets->{'entryProgramsMPlayer'}->signal_connect( activate => \&entryProgramsMPlayerActivated, $widgets );
	$widgets->{'entryProgramsMEncoder'}->signal_connect( activate => \&entryProgramsMEncoderActivated, $widgets );
	$widgets->{'chkbtnSettingsDebug'}->signal_connect( toggled => \&chkbtnSettingsDebugToggled, $widgets );
	## START / STOP
	$widgets->{'btnStopEncoding'}->signal_connect( clicked => \&btnStopEncodingClicked, $widgets );
	$widgets->{'btnStartEncoding'}->signal_connect( clicked => \&btnStartEncodingClicked, $widgets );



	new_debug_window( $widgets );
	new_batch_window( $widgets );

	bless( $widgets, $class );
	return $widgets;
}

sub new_batch_window
{
	my ($widgets) = @_;
	## main window
	$widgets->{'wdwMainBatch'} = Gtk2::Window->new;
	$widgets->{'wdwMainBatch'}->set_title ( 'File2DiVX3Pass '.VERSION.' Batch window' );
	$widgets->{'wdwMainBatch'}->set_border_width( 3 );
	$widgets->{'wdwMainBatch'}->set_transient_for( undef );
	$widgets->{'wdwMainBatch'}->signal_connect( delete_event => sub
												{
													$widgets->{'wdwMainBatch'}->hide_all;
													printMsg( $widgets, 'Closing Batch Encoding' );
													return TRUE; ## wdwMainBatch doesn't get destroyed this way
												}
										);


	## main vbox, top=1.batch, bottom=2.close button
	$widgets->{'vbMainBatch'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'wdwMainBatch'}->add( $widgets->{'vbMainBatch'} );

	## batch
	$widgets->{'hbBatchListButtons'} = Gtk2::HBox->new( FALSE, 1 );
	$widgets->{'hbBatchListButtons'}->set_border_width( 3 );
	$widgets->{'vbMainBatch'}->pack_start( $widgets->{'hbBatchListButtons'}, FALSE, TRUE, 1 );
		## list
		$widgets->{'swBatchList'} = Gtk2::ScrolledWindow->new( undef, undef );
		$widgets->{'swBatchList'}->set_policy( 'automatic', 'always' );
	$widgets->{'hbBatchListButtons'}->pack_start( $widgets->{'swBatchList'}, FALSE, TRUE, 1 );
		$widgets->{'listBatchList'} = Gtk2::SimpleList->new( 'Projects' => 'text', 'Status' => 'text', 'Path' => 'text' );
		$widgets->{'listBatchList'}->get_selection->set_mode('multiple');
		$widgets->{'listBatchList'}->set_rules_hint( TRUE );
		$widgets->{'swBatchList'}->add_with_viewport( $widgets->{'listBatchList'} );
		$widgets->{'swBatchList'}->set_size_request( 480, 240 );

		## buttons
		$widgets->{'vbBatchButtons'} = Gtk2::VBox->new( FALSE, 1 );
		$widgets->{'vbBatchButtons'}->set_border_width( 3 );
	$widgets->{'hbBatchListButtons'}->pack_start( $widgets->{'vbBatchButtons'}, FALSE, TRUE, 1 );
			$widgets->{'btnBatchButtonsAddProject'} = Gtk2::Button->new_from_stock( 'f2d3p-batchaddproject' );
			$widgets->{'btnBatchButtonsRemoveProject'} = Gtk2::Button->new_from_stock( 'f2d3p-batchremoveproject' );
			$widgets->{'btnBatchButtonsClearProjects'} = Gtk2::Button->new_from_stock( 'f2d3p-batchclearprojects' );
			$widgets->{'btnBatchButtonsStart'} = Gtk2::Button->new_from_stock( 'f2d3p-batchstart' );
			$widgets->{'btnBatchButtonsStop'} = Gtk2::Button->new_from_stock( 'f2d3p-batchstop' );
			$widgets->{'btnBatchButtonsStop'}->set_sensitive( FALSE );
			$widgets->{'vbBatchButtons'}->pack_start( $widgets->{'btnBatchButtonsAddProject'}, FALSE, TRUE, 1 );
			$widgets->{'vbBatchButtons'}->pack_start( $widgets->{'btnBatchButtonsRemoveProject'}, FALSE, TRUE, 1 );
			$widgets->{'vbBatchButtons'}->pack_start( $widgets->{'btnBatchButtonsClearProjects'}, FALSE, TRUE, 1 );
			$widgets->{'vbBatchButtons'}->pack_end( $widgets->{'btnBatchButtonsStop'}, FALSE, TRUE, 1 );
			$widgets->{'vbBatchButtons'}->pack_end( $widgets->{'btnBatchButtonsStart'}, FALSE, TRUE, 1 );

	$widgets->{'vbMainBatch'}->pack_start( Gtk2::HSeparator->new, FALSE, TRUE, 1 );

	## progress
	$widgets->{'pbarBatchBar'} = Gtk2::ProgressBar->new;
	$widgets->{'vbMainBatch'}->pack_start( $widgets->{'pbarBatchBar'}, FALSE, TRUE, 1 );
		$widgets->{'pbarBatchBar'}->set_orientation('left_to_right');
		$widgets->{'pbarBatchBar'}->set_text( '0 %' );

	$widgets->{'vbMainBatch'}->pack_start( Gtk2::HSeparator->new, FALSE, TRUE, 1 );

	##
	## close button
	$widgets->{'hbBatchCloseButton'} = Gtk2::HBox->new( FALSE, 1 );
	$widgets->{'hbBatchCloseButton'}->set_border_width( 1 );
	$widgets->{'vbMainBatch'}->pack_start( $widgets->{'hbBatchCloseButton'}, FALSE, FALSE, 1 );
		$widgets->{'btnBatchCloseButton'} = Gtk2::Button->new_from_stock( 'f2d3p-debugclosebutton' );
		$widgets->{'btnBatchCloseButton'}->signal_connect( clicked => sub
														{
															$widgets->{'wdwMainBatch'}->hide_all;
															printMsg( $widgets, 'Closing Batch Encoding' );
															return TRUE; ## wdwMainBatch doesn't get destroyed this way
														}
            );
	$widgets->{'hbBatchCloseButton'}->pack_end( $widgets->{'btnBatchCloseButton'}, FALSE, TRUE, 1 );

	## signals
	$widgets->{'btnBatchButtonsAddProject'}->signal_connect( clicked => \&btnBatchButtonsAddProjectClicked, $widgets );
	$widgets->{'btnBatchButtonsRemoveProject'}->signal_connect( clicked => \&btnBatchButtonsRemoveProjectClicked, $widgets );
	$widgets->{'btnBatchButtonsClearProjects'}->signal_connect( clicked => \&btnBatchButtonsClearProjectsClicked, $widgets );
	$widgets->{'btnBatchButtonsStart'}->signal_connect( clicked => \&btnBatchButtonsStartClicked, $widgets );
	$widgets->{'btnBatchButtonsStop'}->signal_connect( clicked => \&btnBatchButtonsStopClicked, $widgets );

}

sub new_debug_window
{
	my ($widgets) = @_;

	## main window
	$widgets->{'wdwMainDebug'} = Gtk2::Window->new;
	$widgets->{'wdwMainDebug'}->set_title ( 'File2DiVX3Pass '.VERSION.' Debug window' );
	$widgets->{'wdwMainDebug'}->set_border_width( 3 );
	$widgets->{'wdwMainDebug'}->set_transient_for( undef );
	$widgets->{'wdwMainDebug'}->signal_connect( delete_event => sub
												{
													$widgets->{'wdwMainDebug'}->hide_all;
													$widgets->{'chkbtnSettingsDebug'}->set_active( FALSE );
													return TRUE; ## wdwMainDebug doesn't get destroyed this way
												}
										);

	## main vbox, top=1.text, bottom=2.close button
	$widgets->{'vbMainDebug'} = Gtk2::VBox->new( FALSE, 3 );
	$widgets->{'wdwMainDebug'}->add( $widgets->{'vbMainDebug'} );

	## text viewer
	$widgets->{'swMainDebug'} = Gtk2::ScrolledWindow->new;
	$widgets->{'tvMainDebug'} = Gtk2::TextView->new;
	$widgets->{'tvMainDebug'}->set_editable( FALSE );
	$widgets->{'swMainDebug'}->add_with_viewport( $widgets->{'tvMainDebug'} );
	$widgets->{'swMainDebug'}->set_size_request( 480, 240 );
	$widgets->{'vbMainDebug'}->pack_start( $widgets->{'swMainDebug'}, TRUE, TRUE, 1 );


	$widgets->{'vbMainDebug'}->pack_start( Gtk2::HSeparator->new, FALSE, TRUE, 1 );

	##
	## close button
	$widgets->{'hbDebugCloseButton'} = Gtk2::HBox->new( FALSE, 1 );
	$widgets->{'hbDebugCloseButton'}->set_border_width( 1 );
	$widgets->{'vbMainDebug'}->pack_start( $widgets->{'hbDebugCloseButton'}, FALSE, FALSE, 1 );
		$widgets->{'btnDebugCloseButton'} = Gtk2::Button->new_from_stock( 'f2d3p-debugclosebutton' );
		$widgets->{'btnDebugCloseButton'}->signal_connect( clicked => sub
														{
															$widgets->{'wdwMainDebug'}->hide_all;
															$widgets->{'chkbtnSettingsDebug'}->set_active( FALSE );
															return TRUE; ## wdwMainDebug doesn't get destroyed this way
														}
            );
	$widgets->{'hbDebugCloseButton'}->pack_end( $widgets->{'btnDebugCloseButton'}, FALSE, TRUE, 1 );


}


1;
