#!/usr/bin/perl -w

##  Copyright 2004 Rien Croonenborghs
##
##    This file is part of file2divx3pass.
##    file2divx4pass is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##    file2divx3pass is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    You should have received a copy of the GNU General Public License
##    along with file2divx3pass; if not, write to the Free Software
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package File2DiVX3Pass2;
use strict;
use File2DiVX3Pass2::Helpers;


## INPUT-OUTPUT TAB ------------------------------------------------------------

## input file button clicked
## let user select the input file
sub btnIOInputClicked
{
	my ($btn,$widgets) = @_;

	printMsg( $widgets, 'Selecting input file' );

	## create a file selection widgets + add signals
	$widgets->{'fsInput'} = Gtk2::FileSelection->new( 'Select input file' );
	$widgets->{'fsInput'}->signal_connect( close => \&fsInputClose, $widgets );
	$widgets->{'fsInput'}->ok_button->signal_connect( clicked => \&fsInputOkButtonClicked, $widgets );
	$widgets->{'fsInput'}->cancel_button->signal_connect( clicked => \&fsInputCancelButtonClicked, $widgets );
	$widgets->{'fsInput'}->show;
}
## close the file selection widgets for input files
sub fsInputClose
{
	my ($fsel,$widgets) = @_;
	$widgets->{'fsInput'}->destroy;
	printMsg( $widgets, 'Selecting input file cancelled' );
}
## cancel clicked for file selection for input files
sub fsInputCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsInput'}->destroy;
	printMsg( $widgets, 'Selecting input file cancelled' );
}
## ok clicked for file selection for input files
sub fsInputOkButtonClicked
{
	my ($button,$widgets) = @_;
	## get filename
	$widgets->{'fsInput'}->hide();
	my $filename = $widgets->{fsInput}->get_filename();
	$widgets->{'fsInput'}->destroy();

	setInputFile( $filename, $widgets );
}

## identify file with mplayer
## set output filename
## set video length, calc video bitrate and BPP
sub setInputFile
{
	my ($filename,$widgets) = @_;

	printMsg( $widgets, 'Checking file...' );
	## get the specs of the selected files
	if( !readFileSpecs( $filename, $widgets ) )
	{
		printError( $widgets, getError() );
		return;
	}

	## check the size of the CD
	my $cdsize = $widgets->{'entryVideoCDSize'}->get_text;
	if( !txtIsNumeral( $cdsize ) )
	{
		printError( $widgets, getError().' for CD Size') ;
		return;
	}

	## get the audio bitrate
	my $audiobrate = getSetting( 'mp3bitrate' );

	## get fps
	my $fps = getFileSpec( 'fps' );;
	## set output FPS to spec FPS, even if the checkbutton is not active
	$widgets->{'entryVideoOutputFPS'}->set_text( $fps );

	## update input & output fields
	$widgets->{'entryIOInput'}->set_text( $filename );
	my $f = substr( $filename, rindex($filename,"/")+1 );
	$f = substr( $f, 0, rindex($f,".") ).".avi";
	$widgets->{'entryIOOutput'}->set_text( $f );

	## put time in Video Length
	my $seconds = getFileSpec( 'length' );
	$widgets->{'entryVideoLength'}->set_text( StoH( $seconds ) );

	## calculate video bitrate
	my $videobrate = calculateVideoBrate( $cdsize, $seconds/60, $audiobrate );
	printMsg( $widgets, 'Calculated video bitrate is '.$videobrate );

	## put the video brate in widget entryVideoBrate
	$widgets->{'entryVideoBitrate'}->set_text( $videobrate );


	printMsg( $widgets, 'Checking file...done' );
}



## SUBTITLES TAB ---------------------------------------------------------------

## subtitle option changed
## no: all sensitive = 0;
## from file: this = 1, from sid = 0
## from sid: this = 1, from file = 0
sub menitSubtitleTypeActivated
{
	my ($menit,$widgets) = @_;

	if( $widgets->{'optmenSubtitleType'}->get_history == 0 )		# no subtitles
	{
		printMsg( $widgets, "Subtitles set to 'no'" );
		setSetting( 'subtype', 'no' );
		$widgets->{'lblSubtitleFromFile'}->set_sensitive( FALSE );
		$widgets->{'entrySubtitleFromFile'}->set_sensitive( FALSE );
		$widgets->{'btnSubtitleFromFile'}->set_sensitive( FALSE );
		$widgets->{'lblSubtitleFromSID'}->set_sensitive( FALSE );
		$widgets->{'spinSubtitleFromSID'}->set_sensitive( FALSE );
	}
	elsif( $widgets->{'optmenSubtitleType'}->get_history == 1 )		# subs from file
	{
		printMsg( $widgets, "Subtitles set to 'from file'" );
		setSetting( 'subtype', 'from file' );
		$widgets->{'lblSubtitleFromFile'}->set_sensitive( TRUE );
		$widgets->{'entrySubtitleFromFile'}->set_sensitive( TRUE );
		$widgets->{'btnSubtitleFromFile'}->set_sensitive( TRUE );
		$widgets->{'lblSubtitleFromSID'}->set_sensitive( FALSE );
		$widgets->{'spinSubtitleFromSID'}->set_sensitive( FALSE );
	}
	elsif( $widgets->{'optmenSubtitleType'}->get_history == 2 )		# subs from SID
	{
		printMsg( $widgets, "Subtitles set to 'from SID'" );
		setSetting( 'subtype', 'from SID' );
		$widgets->{'lblSubtitleFromFile'}->set_sensitive( FALSE );
		$widgets->{'entrySubtitleFromFile'}->set_sensitive( FALSE );
		$widgets->{'btnSubtitleFromFile'}->set_sensitive( FALSE );
		$widgets->{'lblSubtitleFromSID'}->set_sensitive( TRUE );
		$widgets->{'spinSubtitleFromSID'}->set_sensitive( TRUE );
	}
}

## SID changed
sub adjSubtitleFromSIDValueChanged
{
	my ($adj,$widgets) = @_;

	my $sid = $widgets->{'spinSubtitleFromSID'}->get_value_as_int;
	printMsg( $widgets, 'SID set to '.$sid );
	setSetting( 'sid', $sid );
}

## set the subtitle file
sub btnSubtitleFromFileClicked
{
	my ($button,$widgets) = @_;

	printMsg( $widgets, 'Selecting subtitle file' );

	## create a file selection widgets + add signals
	$widgets->{'fsSubFile'} = new Gtk2::FileSelection( 'Select a subtitle file' );
	$widgets->{'fsSubFile'}->signal_connect( close => \&fsSubFileClose, $widgets );
	$widgets->{'fsSubFile'}->ok_button->signal_connect( clicked => \&fsSubFileOkButtonClicked, $widgets );
	$widgets->{'fsSubFile'}->cancel_button->signal_connect( clicked => \&fsSubFileCancelButtonClicked, $widgets );
	$widgets->{'fsSubFile'}->show;
}
## close the file selection widgets for subtitle file
sub fsSubFileClose
{
	my ($fsel,$widgets) = @_;
	$widgets->{'fsSubFile'}->destroy;
	printMsg( $widgets, 'Selecting subtitle file cancelled' );
}
## cancel clicked for file selection for subtitle file
sub fsSubFileCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSubFile'}->destroy;
	printMsg( $widgets, 'Selecting subtitle file cancelled' );
}
## ok clicked for file selection for subtitle file
sub fsSubFileOkButtonClicked
{
	my ($button,$widgets) = @_;
	## get filename
	$widgets->{'fsSubFile'}->hide();
	my $filename = $widgets->{fsSubFile}->get_filename();
	$widgets->{'fsSubFile'}->destroy();

	## set filename in entry
	$widgets->{'entrySubtitleFromFile'}->set_text( $filename );

	printMsg( $widgets, 'Selecting subtitle file done' );
}


## AUDIO TAB -------------------------------------------------------------------

## audio codec changed
## only display MP3 settings if MP3 codec is selected
## -> change video bitrate
sub menitAudioCodecActivated
{
	my ($optmen,$widgets) = @_;

	if( $widgets->{'optmenAudioCodec'}->get_history == 0 ) # MP3
	{
		printMsg( $widgets, "Audio Codec set to 'MP3'" );
		setSetting( 'audiocodec', 'MP3' );
		setSetting( 'mp3bitrate', getSetting( 'mp3bitratesaved' ) ); ## restore old mp3 bitrate
		$widgets->{'lblAudioMP3'}->set_sensitive( TRUE );
		$widgets->{'optmenAudioMP3Rate'}->set_sensitive( TRUE );
		$widgets->{'lblAudioMP3at'}->set_sensitive( TRUE );
		$widgets->{'optmenAudioMP3Bitrate'}->set_sensitive( TRUE );
		$widgets->{'lblAudioMP3kps'}->set_sensitive( TRUE );
	}
	elsif( $widgets->{'optmenAudioCodec'}->get_history == 1 ) # copy
	{
		printMsg( $widgets, "Audio Codec set to 'copy'" );
		setSetting( 'audiocodec', 'copy' );
		setSetting( 'mp3bitrate', getFileSpec( 'audiobitrate' ) / 1000 ); ## get file specs audiobitrate
		$widgets->{'lblAudioMP3'}->set_sensitive( FALSE );
		$widgets->{'optmenAudioMP3Rate'}->set_sensitive( FALSE );
		$widgets->{'lblAudioMP3at'}->set_sensitive( FALSE );
		$widgets->{'optmenAudioMP3Bitrate'}->set_sensitive( FALSE );
		$widgets->{'lblAudioMP3kps'}->set_sensitive( FALSE );
	}
	elsif( $widgets->{'optmenAudioCodec'}->get_history == 2 ) # PCM
	{
		printMsg( $widgets, "Audio Codec set to 'PCM'" );
		setSetting( 'audiocodec', 'PCM' );
		setSetting( 'mp3bitrate', 192 ); ## 192 kb/s is PCM sound
		$widgets->{'lblAudioMP3'}->set_sensitive( FALSE );
		$widgets->{'optmenAudioMP3Rate'}->set_sensitive( FALSE );
		$widgets->{'lblAudioMP3at'}->set_sensitive( FALSE );
		$widgets->{'optmenAudioMP3Bitrate'}->set_sensitive( FALSE );
		$widgets->{'lblAudioMP3kps'}->set_sensitive( FALSE );
	}

	entryVideoLengthActivated( undef, $widgets );
}

## mp3 rate changed
sub menitAudioMP3RateActivated
{
	my ($menit,$widgets) = @_;

	if( $widgets->{'optmenAudioMP3Rate'}->get_history == 0 ) # constant
	{
		printMsg( $widgets, "MP3 bitrate set to 'constant'" );
		setSetting( 'mp3rate', 'constant' );
	}
	elsif( $widgets->{'optmenAudioMP3Rate'}->get_history == 1 ) # average
	{
		printMsg( $widgets, "MP3 bitrate set to 'average'" );
		setSetting( 'mp3rate', 'average' );
	}
}

## mp3 bitrate changed
## -> change video bitrate
sub menitAudioMP3BitrateActivated
{
	my ($menit,$widgets) = @_;

	my @bitrates = getMP3Bitrates();
	my $mp3br = $bitrates[ $widgets->{'optmenAudioMP3Bitrate'}->get_history ];

	printMsg( $widgets, 'MP3 bitrate set to '.$mp3br.' kb/s' );
	setSetting( 'mp3bitrate', $mp3br );
	setSetting( 'mp3bitratesaved', $mp3br );

	entryVideoLengthActivated( undef, $widgets );
}

## toggle use AID
sub chkbtnAudioAIDToggled
{
	my ($chkbtn,$widgets) = @_;
	if( $widgets->{'chkbtnAudioAID'}->get_active )
	{
		$widgets->{'spinAudioAID'}->set_sensitive( TRUE );
		printMsg( $widgets, 'Enabling AID' );
		setSetting( 'aidset', TRUE );
	}
	else
	{
		$widgets->{'spinAudioAID'}->set_sensitive( FALSE );
		printMsg( $widgets, 'Disabling AID' );
		setSetting( 'aidset', FALSE );
	}
}

## AID changed
sub adjAudioAIDValueChanged
{
	my ($adj,$widgets) = @_;

	my $aid = $widgets->{'spinAudioAID'}->get_value_as_int;
	printMsg( $widgets, 'AID set to $aid' );
	setSetting( 'aid', $aid );
}

## VIDEO TAB -------------------------------------------------------------------

## video length changed
## -> change video bitrate
sub entryVideoLengthActivated
{
	my ($entry,$widgets) = @_;

	## get minutes
	my $minutes = $widgets->{'entryVideoLength'}->get_text;
	if( !txtIsHour( $minutes ) )
	{
		printError( $widgets, getError().' for Video Length' );
		return;
	}
	$minutes = HtoS( $minutes ) / 60;
	## get cdsize
	my $cdsize = $widgets->{'entryVideoCDSize'}->get_text;
	if( !txtIsNumeral( $cdsize ) )
	{
		printError( $widgets, getError().' for CD Size' );
		return;
	}
	## get the audio bitrate
	my $audiobrate = getSetting( 'mp3bitrate' );

	my $videobrate = calculateVideoBrate( $cdsize, $minutes, $audiobrate );
	## put the video brate in widget entryVideoBrate
	$widgets->{'entryVideoBitrate'}->set_text( $videobrate );
}

## video bitrate changed
sub entryVideoBitrateChanged
{
	my ($entry,$widgets) = @_;

	## check output fps
	## from spec or checked from the entry
	my $fps = getFileSpec( 'fps' );;
	if( $widgets->{'chkbtnVideoOutputFPS'}->get_active )
	{
		$fps = $widgets->{'entryVideoOutputFPS'}->get_text;
	}
	if( !txtIsNumeralDecimal( $fps ) )
	{
		printError( $widgets, getError().' for Output FPS' );
		return;
	}
	## check scale width
	my $width = $widgets->{'entryVideoScaleW'}->get_text;
	if( !txtIsNumeral( $width ) )
	{
		printError( $widgets, getError().' for Scale width' );
		return;
	}
	## check scale height
	my $height = $widgets->{'entryVideoScaleH'}->get_text;
	if( !txtIsNumeral( $height ) )
	{
		printError( $widgets, getError().' for Scale height' );
		return;
	}
	## check video bitrate
	my $vbrate = $widgets->{'entryVideoBitrate'}->get_text;
	if( !txtIsNumeral( $vbrate ) )
	{
		printError( $widgets, getError().' for Video bitrate' );
		return;
	}

	my $bpp = calculateBitsPerPixel( $vbrate, $width, $height, $fps );
	$bpp = substr( $bpp, 0, index($bpp,".")+4);
	printMsg( $widgets, 'Bits per Pixel is '.$bpp );
	## put bpp in widget entryVideoBPP
	$widgets->{'entryVideoBPP'}->set_text( $bpp );

}

## toggle video start time
sub chkbtnVideoStartToggled
{
	my ($chkbtn,$widgets) = @_;
	if( $widgets->{'chkbtnVideoStart'}->get_active )
	{
		printMsg( $widgets, 'Enabling encoding from start' );
	}
	else
	{
		printMsg( $widgets, 'Disabling encoding from start' );
	}
}
## entered video start time
sub entryVideoStartActivated
{
	my ($entry,$widgets) = @_;

	if( !txtIsHour( $widgets->{'entryVideoStart'}->get_text ) )
	{
		printError( $widgets, getError().' for Video start' );
		return;
	}
	printMsg( $widgets, "Video start set to '".$widgets->{'entryVideoStart'}->get_text."'" );
}

## toggle video end time
sub chkbtnVideoEndToggled
{
	my ($chkbtn,$widgets) = @_;
	if( $widgets->{'chkbtnVideoEnd'}->get_active )
	{
		printMsg( $widgets, 'Enabling encoding to end' );
	}
	else
	{
		printMsg( $widgets, 'Disabling encoding to end' );
	}
}
## entered video end size
sub entryVideoEndActivated
{
	my ($entry,$widgets) = @_;

	if( !txtIsNumeral( $widgets->{'entryVideoEnd'}->get_text ) )
	{
		printError( $widgets, getError().' for Video end' );
		return;
	}
	printMsg( $widgets, "Video end set to ".$widgets->{'entryVideoEnd'}->get_text." Mb" );
}

## CD size changed
## -> change video bitrate
sub entryVideoCDSizeActivated
{
	my ($entry,$widgets) = @_;

	## get cd size
	my $cdsize = $widgets->{'entryVideoCDSize'}->get_text;
	if( !txtIsNumeral( $cdsize ) )
	{
		printError( $widgets, getError().' for CD Size' );
		return;
	}
	printMsg( $widgets, 'CD Size changed to '.$cdsize );
	setSetting( 'cdsize', $cdsize );
	entryVideoLengthActivated( undef, $widgets );
}

## toggle output FPS
## -> change bits / pixel
sub chkbtnVideoOutputFPSToggled
{
	my ($chkbtn,$widgets) = @_;

	if( $widgets->{'chkbtnVideoOutputFPS'}->get_active )
	{
		$widgets->{'entryVideoOutputFPS'}->set_sensitive( TRUE );
		printMsg( $widgets, 'Enabling Output FPS' );
		setSetting( 'fpsset', TRUE );
	}
	else
	{
		$widgets->{'entryVideoOutputFPS'}->set_sensitive( FALSE );
		printMsg( $widgets, 'Disabling Output FPS' );
		setSetting( 'fpsset', FALSE );
	}

	entryVideoBitrateChanged( undef, $widgets );
}

## output FPS entry changed
## -> change bits / pixel
sub entryVideoOutputFPSActivated
{
	my ($entry,$widgets) = @_;

	my $ofps = $widgets->{'entryVideoOutputFPS'}->get_text;
	if( !txtIsNonNumeral( $ofps ) )
	{
		printError( $widgets, getError().' for Output FPS' );
		return;
	}
	entryVideoBitrateChanged( undef, $widgets );
	setSetting( 'ofps', $ofps );
}

## video codec changed
sub menitVideoCodecActivated
{
	my ($menit,$widgets) = @_;

	if( $widgets->{'optmenVideoCodec'}->get_history == 0 ) # lavc
	{
		printMsg( $widgets, "Video Codec set to 'lavc'" );
		setSetting( 'videocodec', 'lavc' );
		$widgets->{'entryVideoEncOpts'}->set_text( getSetting( 'lavcopts' ) );

		$widgets->{'entryVideoLength'}->set_sensitive( TRUE );
		$widgets->{'entryVideoCDSize'}->set_sensitive( TRUE );
		$widgets->{'chkbtnVideoOutputFPS'}->set_sensitive( TRUE );
		$widgets->{'entryVideoOutputFPS'}->set_sensitive( TRUE );
		$widgets->{'entryVideoScaleW'}->set_sensitive( TRUE );
		$widgets->{'entryVideoScaleH'}->set_sensitive( TRUE );
		$widgets->{'rbtnVideoRation43'}->set_sensitive( TRUE );
		$widgets->{'rbtnVideoRation169'}->set_sensitive( TRUE );
		$widgets->{'entryVideoBitrate'}->set_sensitive( TRUE );
		$widgets->{'entryVideoBPP'}->set_sensitive( TRUE );
		$widgets->{'optmenVideoPass'}->set_sensitive( TRUE );
		$widgets->{'entryVideoEncOpts'}->set_sensitive( TRUE );
		$widgets->{'entryVideoFilterOpts'}->set_sensitive( TRUE );
	}
	elsif( $widgets->{'optmenVideoCodec'}->get_history == 1 ) # divx4
	{
		printMsg( $widgets, "Video Codec set to 'divx4'" );
		setSetting( 'videocodec', 'divx4' );
		$widgets->{'entryVideoEncOpts'}->set_text( getSetting( 'divx4opts' ) );

		$widgets->{'entryVideoLength'}->set_sensitive( TRUE );
		$widgets->{'entryVideoCDSize'}->set_sensitive( TRUE );
		$widgets->{'chkbtnVideoOutputFPS'}->set_sensitive( TRUE );
		$widgets->{'entryVideoOutputFPS'}->set_sensitive( TRUE );
		$widgets->{'entryVideoScaleW'}->set_sensitive( TRUE );
		$widgets->{'entryVideoScaleH'}->set_sensitive( TRUE );
		$widgets->{'rbtnVideoRation43'}->set_sensitive( TRUE );
		$widgets->{'rbtnVideoRation169'}->set_sensitive( TRUE );
		$widgets->{'entryVideoBitrate'}->set_sensitive( TRUE );
		$widgets->{'entryVideoBPP'}->set_sensitive( TRUE );
		$widgets->{'optmenVideoPass'}->set_sensitive( TRUE );
		$widgets->{'entryVideoEncOpts'}->set_sensitive( TRUE );
		$widgets->{'entryVideoFilterOpts'}->set_sensitive( TRUE );
	}
	elsif( $widgets->{'optmenVideoCodec'}->get_history == 2 ) # copy
	{
		printMsg( $widgets, "Video Codec set to 'copy'" );
		setSetting( 'videocodec', 'copy' );

		$widgets->{'entryVideoLength'}->set_sensitive( FALSE );
		$widgets->{'entryVideoCDSize'}->set_sensitive( FALSE );
		$widgets->{'chkbtnVideoOutputFPS'}->set_sensitive( FALSE );
		$widgets->{'entryVideoOutputFPS'}->set_sensitive( FALSE );
		$widgets->{'entryVideoScaleW'}->set_sensitive( FALSE );
		$widgets->{'entryVideoScaleH'}->set_sensitive( FALSE );
		$widgets->{'rbtnVideoRation43'}->set_sensitive( FALSE );
		$widgets->{'rbtnVideoRation169'}->set_sensitive( FALSE );
		$widgets->{'entryVideoBitrate'}->set_sensitive( FALSE );
		$widgets->{'entryVideoBPP'}->set_sensitive( FALSE );
		$widgets->{'optmenVideoPass'}->set_sensitive( FALSE );
		$widgets->{'entryVideoEncOpts'}->set_sensitive( FALSE );
		$widgets->{'entryVideoFilterOpts'}->set_sensitive( FALSE );
	}
}

## passes changed
sub menitVideoPassActivated
{
	my ($menit,$widgets) = @_;

	if( $widgets->{'optmenVideoPass'}->get_history == 0 ) # 1 pass
	{
		printMsg( $widgets, 'Video passes changed to 1' );
		setSetting( 'passes', 1 );
	}
	elsif( $widgets->{'optmenVideoPass'}->get_history == 1 ) # 2 pass
	{
		printMsg( $widgets, 'Video passes changed to 2' );
		setSetting( 'passes', 2 );
	}
	elsif( $widgets->{'optmenVideoPass'}->get_history == 2 ) # 3 pass
	{
		printMsg( $widgets, 'Video passes changed to 3' );
		setSetting( 'passes', 3 );
	}
}

## scale width changed
## -> change bits / pixel
sub entryVideoScaleWActivated
{
	my ($entry,$widgets) = @_;

	my $width = $widgets->{'entryVideoScaleW'}->get_text;
	if( !txtIsNumeral( $width ) )
	{
		printError( $widgets, getError().' for Scale width' );
		return;
	}
	printMsg( $widgets, 'Scale width changed to '.$width );
	setSetting( 'scalew', $width );

	if( $widgets->{'rbtnVideoRation43'}->get_active ) ## update height using 4:3
	{
		my $height = ( $width / 4 ) * 3;
		$widgets->{'entryVideoScaleH'}->set_text( $height );
		printMsg( $widgets, 'Scale: updated height to '.$height );
		setSetting( 'scaleh', $height );
	}
	else									## update height using 16:9
	{
		my $height = ( $width / 16 ) * 9;
		$widgets->{'entryVideoScaleH'}->set_text( $height );
		printMsg( $widgets, 'Scale: updated height to '.$height );
		setSetting( 'scaleh', $height );
	}

	entryVideoBitrateChanged( undef, $widgets );
}

## scale height changed
## -> change bits / pixel
sub entryVideoScaleHActivated
{
	my ($entry,$widgets) = @_;

	my $height = $widgets->{'entryVideoScaleH'}->get_text;
	if( !txtIsNumeral( $height ) )
	{
		printError( $widgets, getError().' for Scale height' );
		return;
	}
	printMsg( $widgets, 'Scale height changed to '.$height );
	setSetting( 'scaleh', $height );

	if( $widgets->{'rbtnVideoRation43'}->get_active ) ## update width using 4:3
	{
		my $width = ( $height / 3 ) * 4;
		$widgets->{'entryVideoScaleW'}->set_text( $width );
		printMsg( $widgets, 'Scale: updated width to '.$width );
		setSetting( 'scalew', $width );
	}
	else									## update width using 16:9
	{
		my $width = ( $height / 9 ) * 16;
		$widgets->{'entryVideoScaleW'}->set_text( $width );
		printMsg( $widgets, 'Scale: updated width to '.$width );
		setSetting( 'scalew', $width );
	}

	entryVideoBitrateChanged( undef, $widgets );
}

## scale ratio changed to/from 4:3
## -> change bits / pixel
sub rbtnVideoRation43Toggled
{
	my ($rbtn,$widgets) = @_;

	printMsg( $widgets, 'Scale ratio toggled' );

	my $width = $widgets->{'entryVideoScaleW'}->get_text;
	if( !txtIsNumeral( $width ) )
	{
		printError( $widgets, getError().' for Scale width' );
		return;
	}
	if( $widgets->{'rbtnVideoRation43'}->get_active ) ## update height using 4:3
	{
		my $height = ( $width / 4 ) * 3;
		$widgets->{'entryVideoScaleH'}->set_text( $height );
		printMsg( $widgets, 'Scale: updated height to '.$height );
		setSetting( 'scaleratio', '43' );
	}
	else									## update height using 16:9
	{
		my $height = ( $width / 16 ) * 9;
		$widgets->{'entryVideoScaleH'}->set_text( $height );
		printMsg( $widgets, 'Scale: updated height to '.$height );
		setSetting( 'scaleratio', '169' );
	}

	entryVideoBitrateChanged( undef, $widgets );
}


## PROGRAMS + SETTINGS TAB -----------------------------------------------------

## mplayer path changed
sub entryProgramsMPlayerActivated
{
	my ($entry,$widgets) = @_;

	my $path = $widgets->{'entryProgramsMPlayer'}->get_text;
	printMsg( $widgets, 'MPlayer path changed to '.$path );
	setSetting( 'mplayerpath', $path );
}

## mencoder path changed
sub entryProgramsMEncoderActivated
{
	my ($entry,$widgets) = @_;

	my $path = $widgets->{'entryProgramsMEncoder'}->get_text;
	printMsg( $widgets, 'MEncoder path changed to '.$path );
	setSetting( 'mencoderpath', $path );
}

## clean up option changed
sub menitSettingsCleanupActivated
{
	my ($menit,$widgets) = @_;

	if( $widgets->{'optmenSettingsCleanup'}->get_history == 0 ) # before
	{
		printMsg( $widgets, "Clean up set to 'before'" );
		setSetting( 'cleanup', 'before' );
	}
	elsif( $widgets->{'optmenSettingsCleanup'}->get_history == 1 ) # after
	{
		printMsg( $widgets, "Clean up set to 'after'" );
		setSetting( 'cleanup', 'after' );
	}
	elsif( $widgets->{'optmenSettingsCleanup'}->get_history == 2 ) # before+after
	{
		printMsg( $widgets, "Clean up set to 'before+after'" );
		setSetting( 'cleanup', 'before+after' );
	}
}

## print debug toggled
sub chkbtnSettingsDebugToggled
{
	my ($chkbtn,$widgets) = @_;

	if( $widgets->{'chkbtnSettingsDebug'}->get_active )
	{
		printMsg( $widgets, 'Enabling debug window' );
		setSetting( 'debug', TRUE );
		$widgets->{'wdwMainDebug'}->show_all;
	}
	else
	{
		printMsg( $widgets, 'Disabling debug window' );
		setSetting( 'debug', FALSE );
		$widgets->{'wdwMainDebug'}->hide_all;
	}
}


## MENU ------------------------------------------------------------------------

## menu NEW PROJECT
sub menuNewProject
{
	my ($widgets) = @_;

	## reset all fields
	## reset input output
	$widgets->{'entryIOInput'}->set_text( '' );
	$widgets->{'entryIOOutput'}->set_text( '' );
	## reset subtitles
	$widgets->{'optmenSubtitleType'}->set_history( 0 ); menitSubtitleTypeActivated( undef, $widgets );
	$widgets->{'entrySubtitleFromFile'}->set_text( '' );
	$widgets->{'spinSubtitleFromSID'}->set_value( 1 );
	## loading the settings will reset most of the audio/video
	menuLoadSettings( $widgets );
	## uncheck AID
	$widgets->{'chkbtnAudioAID'}->set_active( FALSE );
	## reset video length
	$widgets->{'entryVideoLength'}->set_text( '00:00:00' );
	## reset video start
	$widgets->{'chkbtnVideoStart'}->set_active( FALSE );
	$widgets->{'entryVideoStart'}->set_text( '00:00:00' );
	## reset video end
	$widgets->{'chkbtnVideoEnd'}->set_active( FALSE );
	$widgets->{'entryVideoEnd'}->set_text( '700' );
	## reset video bitrate
	$widgets->{'entryVideoBitrate'}->set_text( '0' );
	## reset video bits/pixel
	$widgets->{'entryVideoBPP'}->set_text( '0' );

	printMsg( $widgets, 'New Project' );
}


## menu OPEN PROJECT
sub menuOpenProject
{
	my ($widgets) = @_;

	if( !getSetting( 'isencodingstatus' ) )
	{
		## file selection for opening the project
		$widgets->{'fsOpenProject'} = new Gtk2::FileSelection( 'Open project' );
		$widgets->{'fsOpenProject'}->signal_connect( close => \&fsOpenProjectClose, $widgets );
		$widgets->{'fsOpenProject'}->ok_button->signal_connect( clicked => \&fsOpenProjectOkButtonClicked, $widgets );
		$widgets->{'fsOpenProject'}->cancel_button->signal_connect( clicked => \&fsOpenProjectCancelButtonClicked, $widgets );
		$widgets->{'fsOpenProject'}->show;
	}
}
## close the file selection for opening project
sub fsOpenProjectClose
{
	my ($fsel,$widgets) = @_;
	$widgets->{'fsOpenProject'}->destroy;
	printMsg( $widgets, 'Opening project cancelled' );
}
## cancel opening the project
sub fsOpenProjectCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsOpenProject'}->destroy;
	printMsg( $widgets, 'Opening project cancelled' );
}
## open project
sub fsOpenProjectOkButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsOpenProject'}->hide;
	my $filename = $widgets->{'fsOpenProject'}->get_filename;
	$widgets->{'fsOpenProject'}->destroy;

	readProject( $filename, $widgets );

	printMsg( $widgets, 'Project opened' );
}
## read project file and put in widgets
sub readProject
{
	my ($filename,$widgets) = @_;
	my $res = open(FP,"<$filename");
	if( !defined($res) )
	{
		printError( $widgets, 'Could not open project '.$filename );
		return;
	}
	while( <FP> )
	{
		## input/output
		if( $_ =~ /Input filename=(\S+)/ )
		{
			$widgets->{'entryIOInput'}->set_text( $1 );
			setInputFile( $1, $widgets );
		}
		elsif( $_ =~ /Output filename=(\S+)/ )
		{	$widgets->{'entryIOOutput'}->set_text( $1 );	}
		## subtitles
		elsif( $_ =~ /Subtitle type=(\S+)/ )
		{
			if( $1 eq 'no' )
			{	$widgets->{'optmenSubtitleType'}->set_history( 0 ); setSetting( 'subtype', $1 );	}
			elsif( $1 eq 'file' )
			{	$widgets->{'optmenSubtitleType'}->set_history( 1 ); setSetting( 'subtype', $1 );	}
			if( $1 eq 'SID' )
			{	$widgets->{'optmenSubtitleType'}->set_history( 2 ); setSetting( 'subtype', $1 );	}
			menitSubtitleTypeActivated( undef, $widgets );
		}
		elsif( $_ =~ /Subtitle filename=(\S+)/ )
		{
			$widgets->{'entrySubtitleFromFile'}->set_text( $1 );
		}
		elsif( $_ =~ /Subtitle ID=(\S+)/ )
		{
			$widgets->{'spinSubtitleFromSID'}->set_value( $1 );
			setSetting( 'subid', $1 );
		}
		## audio
		elsif( $_ =~ /Audio codec=(\S+)/ )
		{
			if( $1 eq 'MP3' )
			{	$widgets->{'optmenAudioCodec'}->set_history( 0 ); setSetting( 'audiocodec', $1 ); 	}
			elsif( $1 eq 'copy' )
			{ 	$widgets->{'optmenAudioCodec'}->set_history( 1 ); setSetting( 'audiocodec', $1 ); 	}
			elsif( $1 eq 'PCM' )
			{ 	$widgets->{'optmenAudioCodec'}->set_history( 2 ); setSetting( 'audiocodec', $1 ); 	}
			menitAudioCodecActivated( undef, $widgets );
		}
		elsif( $_ =~ /MP3 rate=(\S+)/ )
		{
			if( $1 eq 'constant' )
			{ 	$widgets->{'optmenAudioMP3Rate'}->set_history( 0 ); setSetting( 'mp3rate', $1 ); 	}
			elsif( $1 eq 'average' )
			{ 	$widgets->{'optmenAudioMP3Rate'}->set_history( 1 ); setSetting( 'mp3rate', $1 ); 	}
			menitAudioMP3RateActivated( undef, $widgets );
		}
		elsif( $_ =~ /MP3 bitrate=(\S+)/ )
		{
			my $count = 0;
			foreach my $mp3br( getMP3Bitrates() )
			{
				if( $mp3br eq $1 )
				{
					$widgets->{'optmenAudioMP3Bitrate'}->set_history( $count );
					setSetting( 'mp3bitrate', $1 );
				}
				else { ++$count; }
			}
			menitAudioMP3BitrateActivated( undef, $widgets );
		}
		elsif( $_ =~ /Audio ID set=(\S+)/ )
		{
			if( $1 eq 'y' )
			{	$widgets->{'chkbtnAudioAID'}->set_active( TRUE ); setSetting( 'aidset', TRUE );	}
			else
			{	$widgets->{'chkbtnAudioAID'}->set_active( FALSE ); setSetting( 'aidset', FALSE );	}
			chkbtnAudioAIDToggled( undef, $widgets );
		}
		elsif( $_ =~ /Audio ID=(\S+)/ )
		{
			$widgets->{'spinAudioAID'}->set_value( $1 );
		}
		## video
		elsif( $_ =~ /Video length=(\S+)/ )
		{
			$widgets->{'entryVideoLength'}->set_text( $1 );
			entryVideoLengthActivated( undef, $widgets );
		}
		elsif( $_ =~ /Video start=(\S+)/ )
		{
			if( $1 eq 'y' )
			{	$widgets->{'chkbtnVideoStart'}->set_active( TRUE );	}
			else
			{	$widgets->{'chkbtnVideoStart'}->set_active( FALSE );	}
			chkbtnVideoStartToggled( undef, $widgets );
		}
		elsif( $_ =~ /Video start time=(\S+)/ )
		{
			$widgets->{'entryVideoStart'}->set_text( $1 );
		}
		elsif( $_ =~ /Video end=(\S+)/ )
		{
			if( $1 eq 'y' )
			{	$widgets->{'chkbtnVideoEnd'}->set_active( TRUE );	}
			else
			{	$widgets->{'chkbtnVideoEnd'}->set_active( FALSE );	}
			chkbtnVideoEndToggled( undef, $widgets );
		}
		elsif( $_ =~ /Video end size=(\S+)/ )
		{
			$widgets->{'entryVideoEnd'}->set_text( $1 );
		}
		elsif( $_ =~ /CD size=(\S+)/ )
		{
			$widgets->{'entryVideoCDSize'}->set_text( $1 );
			setSetting( 'cdsize', $1 );
			entryVideoCDSizeActivated( undef, $widgets );
		}
		elsif( $_ =~ /Output FPS is set=(\S+)/ )
		{
			if( $1 eq 'y' )
			{ $widgets->{'chkbtnVideoOutputFPS'}->set_active( TRUE ); setSetting( 'fpsset', TRUE ); }
			else
			{ $widgets->{'chkbtnVideoOutputFPS'}->set_active( FALSE ); setSetting( 'fpsset', FALSE);  }
			chkbtnVideoOutputFPSToggled( undef, $widgets );
		}
		elsif( $_ =~ /Output FPS=(\S+)/ )
		{
			$widgets->{'entryVideoOutputFPS'}->set_text( $1 );
			setSetting( 'ofps', $1 );
			entryVideoOutputFPSActivated( undef, $widgets );
		}
		elsif( $_ =~ /Scale width=(\S+)/ )
		{
			$widgets->{'entryVideoScaleW'}->set_text( $1 ); setSetting( 'scalew', $1 );
			#entryVideoScaleWActivated( undef, $widgets );
		}
		elsif( $_ =~ /Scale height=(\S+)/ )
		{
			$widgets->{'entryVideoScaleH'}->set_text( $1 ); setSetting( 'scaleh', $1 );
			#entryVideoScaleHActivated( undef, $widgets );
		}
		elsif( $_ =~ /Scale ratio=(\S+)/ )
		{
			if( $1 eq '4:3' ) { $widgets->{'rbtnVideoRation43'}->set_active( TRUE ); setSetting( 'scaleratio', '43' ); }
			else { $widgets->{'rbtnVideoRation169'}->set_active( TRUE ); setSetting( 'scaleratio', '169' ); }
			rbtnVideoRation43Toggled( undef, $widgets );
		}
		elsif( $_ =~ /Video bitrate=(\S+)/ )
		{
			$widgets->{'entryVideoBitrate'}->set_text( $1 );
			entryVideoBitrateChanged( undef, $widgets );
		}
		elsif( $_ =~ /Video bits\/pixel=(\S+)/ )
		{
			$widgets->{'entryVideoBPP'}->set_text( $1 );
		}
		elsif( $_ =~ /Video codec=(\S+)/ )
		{
			if( $1 eq 'lavc' ) { $widgets->{'optmenVideoCodec'}->set_history( 0 ); setSetting( 'videocodec', $1 ); }
			elsif( $1 eq 'divx4' ) { $widgets->{'optmenVideoCodec'}->set_history( 1 ); setSetting( 'videocodec', $1 ); }
			elsif( $1 eq 'copy' ) { $widgets->{'optmenVideoCodec'}->set_history( 2 ); setSetting( 'videocodec', $1 ); }
			menitVideoCodecActivated( undef, $widgets );
		}
		elsif( $_ =~ /Video passes=(\S+)/ )
		{
			if( $1 eq '1' ) { $widgets->{'optmenVideoPass'}->set_history( 0 ); setSetting( 'passes', 1 ); }
			elsif( $1 eq '2' ) { $widgets->{'optmenVideoPass'}->set_history( 1 ); setSetting( 'passes', 2 ); }
			elsif( $1 eq '3' ) { $widgets->{'optmenVideoPass'}->set_history( 2 ); setSetting( 'passes', 3 ); }
			menitVideoPassActivated( undef, $widgets );
		}
		elsif( $_ =~ /Video encoding options lavc=(\S+)/ )
		{
			## update entry if needed, but set the setting!
			if( getSetting( 'videocodec' ) eq 'lavc' )
			{ $widgets->{'entryVideoEncOpts'}->set_text( $1 ); }
			setSetting( 'lavcopts', $1 );
		}
		elsif( $_ =~ /Video encoding options divx4=(\S+)/ )
		{
			## update entry if needed, but set the setting!
			if( getSetting( 'videocodec' ) eq 'divx4' )
			{ $widgets->{'entryVideoEncOpts'}->set_text( $1 ); }
			setSetting( 'divx4opts', $1 );
		}
		elsif( $_ =~ /Video filter options=(\S+)/ )
		{
			$widgets->{'entryVideoFilterOpts'}->set_text( $1 ); setSetting( 'filters', $1 );
		}
	}
	close( FP );
	printMsg( $widgets, 'Project loaded' );
}


## menu SAVE PROJECT
sub menuSaveProject
{
	my ($widgets) = @_;

	if( !getSetting( 'isencodingstatus' ) )
	{
		## start with a file selection where to save to
		$widgets->{'fsSaveProject'} = new Gtk2::FileSelection( 'Save project' );
		$widgets->{'fsSaveProject'}->signal_connect( close => \&fsSaveProjectClose, $widgets );
		$widgets->{'fsSaveProject'}->ok_button->signal_connect( clicked => \&fsSaveProjectOkButtonClicked, $widgets );
		$widgets->{'fsSaveProject'}->cancel_button->signal_connect( clicked => \&fsSaveProjectCancelButtonClicked, $widgets );
		$widgets->{'fsSaveProject'}->show;
	}
}
## close the file selection for saving the project
sub fsSaveProjectClose
{
	my ($fsel,$widgets) = @_;
	$widgets->{'fsSaveProject'}->destroy;
	printMsg( $widgets, 'Saving project cancelled' );
}
## cancel saving the project
sub fsSaveProjectCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSaveProject'}->destroy;
	printMsg( $widgets, 'Saving project cancelled' );
}
## save the project
sub fsSaveProjectOkButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSaveProject'}->hide;
	my $filename = $widgets->{'fsSaveProject'}->get_filename;
	$widgets->{'fsSaveProject'}->destroy;

	my $res = open( FP, ">$filename" );
	if( !defined($res) )
	{
		printError( $widgets, 'Could not save project' );
		return;
	}

	## input/output
	print FP 'Input filename='.$widgets->{'entryIOInput'}->get_text."\n";
	print FP 'Output filename='.$widgets->{'entryIOOutput'}->get_text."\n";
	## subtitles
	if( getSetting( 'subtype' ) eq 'no' )
	{ print FP 'Subtitle type=no'."\n"; }
	elsif( getSetting( 'subtype' ) eq 'from file' )
	{ print FP 'Subtitle type=file'."\n"; }
	elsif( getSetting( 'subtype' ) eq 'from SID' )
	{ print FP 'Subtitle type=SID'."\n"; }
	if( getSetting( 'subtype' ) eq 'from file' )
	{ print FP 'Subtitle filename='.$widgets->{'entrySubtitleFromFile'}->get_text."\n"; }
	elsif( getSetting( 'subtype' ) eq 'from SID' )
	{ print FP 'Subtitle ID='.$widgets->{'spinSubtitleFromSID'}->get_value."\n"; }
	## audio
	print FP 'Audio codec='.getSetting( 'audiocodec' )."\n";
	if( getSetting( 'audiocodec' ) eq 'MP3' )
	{
		print FP 'MP3 rate='.getSetting( 'mp3rate' )."\n";
		print FP 'MP3 bitrate='.getSetting( 'mp3bitrate' )."\n";
	}
	if( $widgets->{'chkbtnAudioAID'}->get_active )
	{
		print  FP 'Audio ID set=y'."\n";
		print  FP 'Audio ID='.$widgets->{'spinAudioAID'}->get_value."\n";
	}
	else { print  FP 'Audio ID set=n'."\n"; }
	## video
	print FP 'Video length='.$widgets->{'entryVideoLength'}->get_text."\n";
	if( $widgets->{'chkbtnVideoStart'}->get_active )
	{
		print FP 'Video start=y'."\n";
		print FP 'Video start time='.$widgets->{'entryVideoStart'}->get_text."\n";
	}
	else { print FP 'Video start=n'."\n"; }
	if( $widgets->{'chkbtnVideoEnd'}->get_active )
	{
		print FP 'Video end=y'."\n";
		print FP 'Video end size='.$widgets->{'entryVideoEnd'}->get_text."\n";
	}
	else { print FP 'Video end=n'."\n"; }
	print FP 'CD size='.getSetting( 'cdsize' )."\n";
	if( getSetting( 'fpsset' ) )
	{
		print FP 'Output FPS is set=y'."\n";
		print FP 'Output FPS='.getSetting( 'ofps' )."\n";
	}
	else
	{ print FP 'Output FPS is set=n'."\n"; }
	print FP 'Scale width='.getSetting( 'scalew' )."\n";
	print FP 'Scale height='.getSetting( 'scaleh' )."\n";
	if( getSetting( 'scaleratio' ) eq '43' )
	{ print FP 'Scale ratio=4:3'."\n"; }
	else
	{ print FP 'Scale ratio=16:9'."\n"; }
	print FP 'Video bitrate='.$widgets->{'entryVideoBitrate'}->get_text."\n";
	print FP 'Video bits/pixel='.$widgets->{'entryVideoBPP'}->get_text."\n";
	print FP 'Video codec='.getSetting( 'videocodec' )."\n";
	print FP 'Video passes='.getSetting( 'passes' )."\n";
	print FP 'Video encoding options lavc='.getSetting( 'lavcopts' )."\n";
	print FP 'Video encoding options divx4='.getSetting( 'divx4opts' )."\n";
	print FP 'Video filter options='.getSetting( 'filters' )."\n";

	close( FP );

	printMsg( $widgets, "Project saved" );
}


## menu SAVE SETTINGS
sub menuSaveSettings
{
	my ($widgets) = @_;

	my $home = $ENV{HOME};
	if( !settingsDirExists() )
	{

		if( !mkdir( $home.'/.file2divx3pass' ) )
		{
			printError( $widgets, 'Could not create dir .file2divx3pass in your home dir' );
			return;
		}
	}

	my $path = $home.'/.file2divx3pass/settings';
	my $res = open( FP, ">$path" );
	if( !defined($res) )
	{
		printError( $widgets, 'Could not write to settings file' );
		return;
	}

	## audio settings
	print FP 'Audio codec='.getSetting( 'audiocodec' )."\n";;
	print FP 'MP3 rate='.getSetting( 'mp3rate' )."\n";
	print FP 'MP3 bitrate='.getSetting( 'mp3bitrate' )."\n";
	## video settings
	print FP 'CD size='.getSetting( 'cdsize' )."\n";
	if( getSetting( 'fpsset' ) )
	{
		print FP 'Output FPS is set=y'."\n";
		print FP 'Output FPS='.getSetting( 'ofps' )."\n";
	}
	else
	{ print FP 'Output FPS is set=n'."\n"; }
	print FP 'Scale width='.getSetting( 'scalew' )."\n";
	print FP 'Scale height='.getSetting( 'scaleh' )."\n";
	if( getSetting( 'scaleratio' ) eq '43' )
	{ print FP 'Scale ratio=4:3'."\n"; }
	else
	{ print FP 'Scale ratio=16:9'."\n"; }
	print FP 'Video codec='.getSetting( 'videocodec' )."\n";
	print FP 'Video passes='.getSetting( 'passes' )."\n";
	print FP 'Video encoding options lavc='.getSetting( 'lavcopts' )."\n";
	print FP 'Video encoding options divx4='.getSetting( 'divx4opts' )."\n";
	print FP 'Video filter options='.getSetting( 'filters' )."\n";
	## program+settings settings
	print FP 'MPlayer path='.getSetting( 'mplayerpath' )."\n";
	print FP 'MEncoder path='.getSetting( 'mencoderpath' )."\n";
	print FP 'Clean up='.getSetting( 'cleanup' )."\n";
	if( getSetting( 'debug' ) )
	{ print FP 'Show debug window=y'."\n"; }
	else
	{ print FP 'Show debug window=n'."\n"; }

	close( FP );
	printMsg( $widgets, "Settings saved" );
}

## menu LOAD SETTINGS
sub menuLoadSettings
{
	my ($widgets) = @_;

	## get homedir
	my $home = $ENV{HOME};
	## if settingsdir not exists error
	if( !settingsDirExists() )
	{
		printError( $widgets, 'Could not find your settings' );
		return;
	}

	my $path = $home.'/.file2divx3pass/settings';
	my $res = open( FP, "<$path");
	if( !defined($res) )
	{
		printError( $widgets, 'Could not read settingsfile' );
		return;
	}
	while( <FP> )
	{
		## audio settings
		if( $_ =~ /Audio codec=(\S+)/ )
		{
			## default to MP3
			if( $1 eq 'MP3' ) { $widgets->{'optmenAudioCodec'}->set_history( 0 ); setSetting( 'audiocodec', $1 ); }
			elsif( $1 eq 'copy' ) { $widgets->{'optmenAudioCodec'}->set_history( 1 ); setSetting( 'audiocodec', $1 ); }
			elsif( $1 eq 'PCM' ) { $widgets->{'optmenAudioCodec'}->set_history( 2 ); setSetting( 'audiocodec', $1 ); }
			else { $widgets->{'optmenAudioCodec'}->set_history( 0 ); setSetting( 'audiocodec', 'MP3' ); }
			menitAudioCodecActivated( undef, $widgets );
		}
		elsif( $_ =~ /MP3 rate=(\S+)/ )
		{
			## default to constant
			if( $1 eq 'constant' ) { $widgets->{'optmenAudioMP3Rate'}->set_history( 0 ); setSetting( 'mp3rate', $1 ); }
			elsif( $1 eq 'average' ) { $widgets->{'optmenAudioMP3Rate'}->set_history( 1 ); setSetting( 'mp3rate', $1 ); }
			else { $widgets->{'optmenAudioMP3Rate'}->set_history( 0 ); setSetting( 'mp3rate', 'constant' ); }
			menitAudioMP3RateActivated( undef, $widgets );
		}
		elsif( $_ =~ /MP3 bitrate=(\S+)/ )
		{
			my $count = 0;
			foreach my $mp3br( getMP3Bitrates() )
			{
				if( $mp3br eq $1 )
				{
					$widgets->{'optmenAudioMP3Bitrate'}->set_history( $count );
					setSetting( 'mp3bitrate', $1 );
				}
				else { ++$count; }
			}
			menitAudioMP3BitrateActivated( undef, $widgets );
		}
		## video settings
		elsif( $_ =~ /CD size=(\S+)/ )
		{
			$widgets->{'entryVideoCDSize'}->set_text( $1 );
			setSetting( 'cdsize', $1 );
			entryVideoCDSizeActivated( undef, $widgets );
		}
		elsif( $_ =~ /Output FPS is set=(\S+)/ )
		{
			if( $1 eq 'y' )
			{ $widgets->{'chkbtnVideoOutputFPS'}->set_active( TRUE ); setSetting( 'fpsset', TRUE ); }
			else
			{ $widgets->{'chkbtnVideoOutputFPS'}->set_active( FALSE ); setSetting( 'fpsset', FALSE);  }
			chkbtnVideoOutputFPSToggled( undef, $widgets );
		}
		elsif( $_ =~ /Output FPS=(\S+)/ )
		{
			$widgets->{'entryVideoOutputFPS'}->set_text( $1 );
			setSetting( 'ofps', $1 );
			entryVideoOutputFPSActivated( undef, $widgets );
		}
		elsif( $_ =~ /Scale width=(\S+)/ )
		{
			$widgets->{'entryVideoScaleW'}->set_text( $1 ); setSetting( 'scalew', $1 );
			entryVideoScaleWActivated( undef, $widgets );
		}
		elsif( $_ =~ /Scale height=(\S+)/ )
		{
			$widgets->{'entryVideoScaleH'}->set_text( $1 ); setSetting( 'scaleh', $1 );
			entryVideoScaleHActivated( undef, $widgets );
		}
		elsif( $_ =~ /Scale ratio=(\S+)/ )
		{
			if( $1 eq '4:3' ) { $widgets->{'rbtnVideoRation43'}->set_active( TRUE ); setSetting( 'scaleratio', '43' ); }
			else { $widgets->{'rbtnVideoRation169'}->set_active( TRUE ); setSetting( 'scaleratio', '169' ); }
		}
		elsif( $_ =~ /Video codec=(\S+)/ )
		{
			## default to lavc
			if( $1 eq 'lavc' ) { $widgets->{'optmenVideoCodec'}->set_history( 0 ); setSetting( 'videocodec', $1 ); }
			elsif( $1 eq 'divx4' ) { $widgets->{'optmenVideoCodec'}->set_history( 1 ); setSetting( 'videocodec', $1 ); }
			elsif( $1 eq 'copy' ) { $widgets->{'optmenVideoCodec'}->set_history( 2 ); setSetting( 'videocodec', $1 ); }
			else { $widgets->{'optmenVideoCodec'}->set_history( 0 ); setSetting( 'videocodec', $1 ); }
			menitVideoCodecActivated( undef, $widgets );
		}
		elsif( $_ =~ /Video passes=(\S+)/ )
		{
			## default to 2
			if( $1 eq '1' ) { $widgets->{'optmenVideoPass'}->set_history( 0 ); setSetting( 'passes', 1 ); }
			elsif( $1 eq '2' ) { $widgets->{'optmenVideoPass'}->set_history( 1 ); setSetting( 'passes', 2 ); }
			elsif( $1 eq '3' ) { $widgets->{'optmenVideoPass'}->set_history( 2 ); setSetting( 'passes', 3 ); }
			else { $widgets->{'optmenVideoPass'}->set_history( 1 ); setSetting( 'passes', 2 ); }
			menitVideoPassActivated( undef, $widgets );
		}
		elsif( $_ =~ /Video encoding options lavc=(\S+)/ )
		{
			## update entry if needed, but set the setting!
			if( getSetting( 'videocodec' ) eq 'lavc' )
			{ $widgets->{'entryVideoEncOpts'}->set_text( $1 ); }
			setSetting( 'lavcopts', $1 );
		}
		elsif( $_ =~ /Video encoding options divx4=(\S+)/ )
		{
			## update entry if needed, but set the setting!
			if( getSetting( 'videocodec' ) eq 'divx4' )
			{ $widgets->{'entryVideoEncOpts'}->set_text( $1 ); }
			setSetting( 'divx4opts', $1 );
		}
		elsif( $_ =~ /Video filter options=(\S+)/ )
		{
			$widgets->{'entryVideoFilterOpts'}->set_text( $1 ); setSetting( 'filters', $1 );
		}
		elsif( $_ =~ /MPlayer path=(\S+)/ )
		{
			$widgets->{'entryProgramsMPlayer'}->set_text( $1 ); setSetting( 'mplayerpath', $1 );
		}
		elsif( $_ =~ /MEncoder path=(\S+)/ )
		{
			$widgets->{'entryProgramsMEncoder'}->set_text( $1 ); setSetting( 'mencoderpath', $1 );
		}
		elsif( $_ =~ /Clean up=(\S+)/ )
		{
			## defaut to before+after
			if( $1 eq 'before' ) { $widgets->{'optmenSettingsCleanup'}->set_history( 0 ); setSetting( 'cleanup', $1 ); }
			elsif( $1 eq 'after' ) { $widgets->{'optmenSettingsCleanup'}->set_history( 1 ); setSetting( 'cleanup', $1 ); }
			elsif( $1 eq 'before+after' ) { $widgets->{'optmenSettingsCleanup'}->set_history( 2 ); setSetting( 'cleanup', $1 ); }
			else { $widgets->{'optmenSettingsCleanup'}->set_history( 2 ); setSetting( 'cleanup', 'before+after' ); }
		}
		elsif( $_ =~ /Show debug window=(\S+)/ )
		{
			if( $1 eq 'y' ) { $widgets->{'chkbtnSettingsDebug'}->set_active( TRUE ); setSetting( 'debug', TRUE ); }
			else { $widgets->{'chkbtnSettingsDebug'}->set_active( FALSE ); setSetting( 'debug', FALSE ); }
		}
	} ## while <FP> loop
	close( FP );

	printMsg( $widgets, 'Settings loaded' );

}

## show the bathc encoding window
sub menuBatchEndcoding
{
	my ($widgets) = @_;

	if( !getSetting( 'isencodingstatus' ) )
	{
		$widgets->{'wdwMainBatch'}->show_all;
		printMsg( $widgets, 'Opening Batch Encoding' );
	}
}
## add project(s) to the list
sub btnBatchButtonsAddProjectClicked
{
	my ($btn,$widgets) = @_;

	## start file selection
	## multiple selection !!
	$widgets->{'fsBatchAddProject'} = Gtk2::FileSelection->new( 'Select project(s)' );
	$widgets->{'fsBatchAddProject'}->signal_connect( close => \&fsBatchAddProjectClose, $widgets );
	$widgets->{'fsBatchAddProject'}->ok_button->signal_connect( clicked => \&fsBatchAddProjectOkButtonClicked, $widgets );
	$widgets->{'fsBatchAddProject'}->cancel_button->signal_connect( clicked => \&fsBatchAddProjectCancelButtonClicked, $widgets );
	$widgets->{'fsBatchAddProject'}->set_select_multiple( TRUE );
	$widgets->{'fsBatchAddProject'}->show;
}
## add project to list closed
sub fsBatchAddProjectClose
{
	my ($filesel,$widgets) = @_;
	$widgets->{'fsBatchAddProject'}->destroy;
	printMsg( $widgets, 'Add project(s) to Batch cancelled' );
}
## cancel adding project to list
sub fsBatchAddProjectCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsBatchAddProject'}->destroy;
	printMsg( $widgets, 'Add project(s) to Batch cancelled' );
}
## add project to list
sub fsBatchAddProjectOkButtonClicked
{
	my ($button,$widgets) = @_;
 	$widgets->{'fsBatchAddProject'}->hide;

	my @filenames = $widgets->{'fsBatchAddProject'}->get_selections;
	if( $#filenames > -1 )
	{
		foreach my $name( @filenames )
		{
			my $filepath = substr($name,0,rindex($name,'/')+1);
			my $filename = substr($name,rindex($name,'/')+1);
			## push a new item at end of the list
			push @{ $widgets->{'listBatchList'}->{data} }, [ $filename, 'queued', $filepath ];
		}
	}
	$widgets->{'fsBatchAddProject'}->destroy;

	$widgets->{pbarBatchBar}->set_text( '0 %' );
	$widgets->{pbarBatchBar}->set_fraction( 0 );


	printMsg( $widgets, 'Project(s) added' );
}

## remove selected items from the list
sub btnBatchButtonsRemoveProjectClicked
{
	my ($btn,$widgets) = @_;

	my @selected = $widgets->{'listBatchList'}->get_selected_indices;
	@selected = sort( @selected ); ## 1,2,3,...,n
	@selected = reverse( @selected ); ## n,n-1,n-2,...,2,1
	foreach my $seli( @selected )
	{
		delete $widgets->{'listBatchList'}->{data}[$seli];
	}

	$widgets->{pbarBatchBar}->set_text( '0 %' );
	$widgets->{pbarBatchBar}->set_fraction( 0 );

	printMsg( $widgets, 'Project(s) removed' );

}
## clear the whole list
sub btnBatchButtonsClearProjectsClicked
{
	my ($btn,$widgets) = @_;

	my $num = $#{ $widgets->{'listBatchList'}->{data} };
	for( my $i=0; $i<$num+1; ++$i )
	{
		delete $widgets->{'listBatchList'}->{data}[0];
	}

	$widgets->{pbarBatchBar}->set_text( '0 %' );
	$widgets->{pbarBatchBar}->set_fraction( 0 );


	printMsg( $widgets, 'Project(s) cleared' );
}
## start bathc encoding
sub btnBatchButtonsStartClicked
{
	my ($button, $widgets) = @_;

	printMsg( $widgets, 'Batch Encoding started' );

	my @projects = @{ $widgets->{'listBatchList'}->{data} };

	if( $#projects > -1 )
	{
		if( !getSetting( 'isencodingstatus' ) )
		{
			$widgets->{'btnBatchButtonsStart'}->set_sensitive( FALSE );
			$widgets->{'btnBatchButtonsStop'}->set_sensitive( TRUE );
			setSetting( 'isbatchencodingstatus', TRUE );

			$widgets->{pbarBatchBar}->set_text( '0 %' );
			$widgets->{pbarBatchBar}->set_fraction( 0 );
			while( Gtk2->events_pending() ) { Gtk2->main_iteration() };

			my $pct = 1 / ($#projects+1);
			for( my $i=0; $i<@projects; ++$i )
			{
				if( getSetting( 'isbatchencodingstatus' ) )
				{
					## get project
					my @project = @{ $widgets->{'listBatchList'}->{data}[$i] };
					printMsg( $widgets, 'Batch encoding '.$project[0] );
					my $projectstatus = $project[1];
					my $projectfullpath = $project[2].'/'.$project[0];

					## only encode queued items
					if( $projectstatus eq 'queued' )
					{
						## start encoding
						## change status to busy
						@{ $widgets->{'listBatchList'}->{data}[$i] }[1] = 'busy';
						readProject( $projectfullpath, $widgets );
						btnStartEncodingClicked( undef, $widgets);
					}
					else
					{
						printMsg( $widgets, 	$project[0].' has status \'done\', skipping' );
					}

					## update progressbar
					my $mypct = $pct * ($i+1);
					my $my100pct = $mypct*100;

					if( $mypct <= 1 )
					{
						$my100pct = substr($my100pct,0,index($my100pct,'.')) if( $my100pct =~ /\./ ); ## locale .
						$my100pct = substr($my100pct,0,index($my100pct,',')) if( $my100pct =~ /\,/ ); ## locale ,

						$widgets->{pbarBatchBar}->set_fraction( $mypct ); #$pct * ($i+1) );
						$widgets->{pbarBatchBar}->set_text( $my100pct.' %' ); #($pct * ($i+1) * 100).' %' );
					}
					else
					{
						$widgets->{pbarBatchBar}->set_fraction( 1 );
						$widgets->{pbarBatchBar}->set_text( '100 %' );
					}
					## change status to done
					@{ $widgets->{'listBatchList'}->{data}[$i] }[1] = 'done';
					while( Gtk2->events_pending() ) { Gtk2->main_iteration() };

				}
				else
				{
					printMsg( $widgets, 'Batch encoding stopped' );
				}
			}
			setSetting( 'isbatchencodingstatus', FALSE );
			$widgets->{'btnBatchButtonsStart'}->set_sensitive( TRUE );
			$widgets->{'btnBatchButtonsStop'}->set_sensitive( FALSE );

		}
	}

	printMsg( $widgets, 'Batch Encoding done' );
}
## stop batch encoding
sub btnBatchButtonsStopClicked
{
	my ($button,$widgets) = @_;
	printMsg( $widgets, 'Stop Batch Encoding' );
	setSetting( 'isbatchencodingstatus', FALSE );
	btnStopEncodingClicked( undef, $widgets );
}


## START / STOP ----------------------------------------------------------------

## start encoding
sub btnStartEncodingClicked
{
	my ($btn,$widgets) = @_;

	## check for an input / output file
	if( $widgets->{'entryIOInput'}->get_text eq '' )
	{ printError( $widgets, 'No input file found. Please select one.' ); return; }
	my $inputfile = $widgets->{'entryIOInput'}->get_text;
	if( $widgets->{'entryIOOutput'}->get_text eq '' )
	{ printError( $widgets, 'No output file found.' ); return; }
	my $outputfile = $widgets->{'entryIOOutput'}->get_text;

	## start encoding if we are not already busy
	if( !getSetting( 'isencodingstatus' ) )
	{
		setSetting( 'isencodingstatus', TRUE );
			$widgets->{'btnStartEncoding'}->set_sensitive( FALSE );
			$widgets->{'btnStopEncoding'}->set_sensitive( TRUE );
			while( Gtk2->events_pending() ) { Gtk2->main_iteration() };
			if( encodeVideo( $inputfile, $outputfile, $widgets ) )
			{
				$widgets->{'btnStartEncoding'}->set_sensitive( TRUE );
				$widgets->{'btnStopEncoding'}->set_sensitive( FALSE );
			}
			else
			{
				printError( $widgets, getError() );
			}
		if( getSetting( 'isencodingstatus' ) )
		{ printMsg( $widgets, 'Encoding done' ); }
		else
		{ printMsg( $widgets, 'Encoding stopped' ); }
		setSetting( 'isencodingstatus', FALSE );
	}
	else
	{ printError( $widgets, 'Already encoding' ); return; }

}

## stop encoding
sub btnStopEncodingClicked
{
	my ($btn,$widgets) = @_;

	if( getSetting( 'isencodingstatus' ) )
	{
		my $pid = getSetting( 'mencoderpid' );
		if( $pid != -1 )
		{
			## kill the menoder PID
			## release buttons
			++$pid;
			kill 9, $pid;
			$widgets->{'btnStartEncoding'}->set_sensitive( TRUE );
			$widgets->{'btnStopEncoding'}->set_sensitive( FALSE );
			while( Gtk2->events_pending() ) { Gtk2->main_iteration() };
			setSetting( 'isencodingstatus', FALSE );
			printMsg( $widgets, 'Encoding stopped' );
		}
	}
}

1;
